/********************************************************************************
** Form generated from reading UI file 'settingsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSDIALOG_H
#define UI_SETTINGSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "gui/qmlutils.h"

namespace OCC {

class Ui_SettingsDialog
{
public:
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout;
    QStackedWidget *dialogStack;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    OCC::QmlUtils::OCQuickWidget *quickWidget;
    QStackedWidget *stack;

    void setupUi(QMainWindow *OCC__SettingsDialog)
    {
        if (OCC__SettingsDialog->objectName().isEmpty())
            OCC__SettingsDialog->setObjectName("OCC__SettingsDialog");
        OCC__SettingsDialog->resize(800, 500);
        centralwidget = new QWidget(OCC__SettingsDialog);
        centralwidget->setObjectName("centralwidget");
        verticalLayout = new QVBoxLayout(centralwidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        dialogStack = new QStackedWidget(centralwidget);
        dialogStack->setObjectName("dialogStack");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        quickWidget = new OCC::QmlUtils::OCQuickWidget(page);
        quickWidget->setObjectName("quickWidget");

        verticalLayout_2->addWidget(quickWidget);

        stack = new QStackedWidget(page);
        stack->setObjectName("stack");

        verticalLayout_2->addWidget(stack);

        dialogStack->addWidget(page);

        verticalLayout->addWidget(dialogStack);

        OCC__SettingsDialog->setCentralWidget(centralwidget);

        retranslateUi(OCC__SettingsDialog);

        QMetaObject::connectSlotsByName(OCC__SettingsDialog);
    } // setupUi

    void retranslateUi(QMainWindow *OCC__SettingsDialog)
    {
        OCC__SettingsDialog->setProperty("Settings", QVariant(QCoreApplication::translate("OCC::SettingsDialog", "MainWindow", nullptr)));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class SettingsDialog: public Ui_SettingsDialog {};
} // namespace Ui
} // namespace OCC

#endif // UI_SETTINGSDIALOG_H
