/********************************************************************************
** Form generated from reading UI file 'folderwizardsourcepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOLDERWIZARDSOURCEPAGE_H
#define UI_FOLDERWIZARDSOURCEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FolderWizardSourcePage
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLineEdit *localFolderLineEdit;
    QPushButton *localFolderChooseBtn;
    QLabel *warnLabel;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *FolderWizardSourcePage)
    {
        if (FolderWizardSourcePage->objectName().isEmpty())
            FolderWizardSourcePage->setObjectName("FolderWizardSourcePage");
        FolderWizardSourcePage->resize(423, 174);
        verticalLayout = new QVBoxLayout(FolderWizardSourcePage);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(FolderWizardSourcePage);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        localFolderLineEdit = new QLineEdit(FolderWizardSourcePage);
        localFolderLineEdit->setObjectName("localFolderLineEdit");

        horizontalLayout->addWidget(localFolderLineEdit);

        localFolderChooseBtn = new QPushButton(FolderWizardSourcePage);
        localFolderChooseBtn->setObjectName("localFolderChooseBtn");

        horizontalLayout->addWidget(localFolderChooseBtn);


        verticalLayout->addLayout(horizontalLayout);

        warnLabel = new QLabel(FolderWizardSourcePage);
        warnLabel->setObjectName("warnLabel");
        warnLabel->setTextFormat(Qt::TextFormat::RichText);

        verticalLayout->addWidget(warnLabel);

        verticalSpacer = new QSpacerItem(349, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(FolderWizardSourcePage);

        QMetaObject::connectSlotsByName(FolderWizardSourcePage);
    } // setupUi

    void retranslateUi(QWidget *FolderWizardSourcePage)
    {
        FolderWizardSourcePage->setWindowTitle(QCoreApplication::translate("FolderWizardSourcePage", "Form", nullptr));
        label->setText(QCoreApplication::translate("FolderWizardSourcePage", "Pick a local folder on your computer to sync", nullptr));
        localFolderChooseBtn->setText(QCoreApplication::translate("FolderWizardSourcePage", "Browse\342\200\246", nullptr));
        warnLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class FolderWizardSourcePage: public Ui_FolderWizardSourcePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FOLDERWIZARDSOURCEPAGE_H
