/****************************************************************************
** Meta object code from reading C++ file 'spacesmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/graphapi/spacesmanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'spacesmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::GraphApi::SpacesManager::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::GraphApi::SpacesManager",
        "spaceChanged",
        "",
        "Space*",
        "space",
        "updated",
        "Account*",
        "account",
        "ready",
        "spaceAdded",
        "QUuid",
        "accountId",
        "OCC::GraphApi::Space*",
        "spaceAboutToBeRemoved",
        "spacesAdded",
        "QList<OCC::GraphApi::Space*>",
        "spaces",
        "totalSpaceCount",
        "spacesRemoved",
        "deletedSpaces"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'spaceChanged'
        QtMocHelpers::SignalData<void(Space *) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'updated'
        QtMocHelpers::SignalData<void(Account *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'ready'
        QtMocHelpers::SignalData<void() const>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spaceAdded'
        QtMocHelpers::SignalData<void(QUuid, OCC::GraphApi::Space *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 4 },
        }}),
        // Signal 'spaceAboutToBeRemoved'
        QtMocHelpers::SignalData<void(QUuid, OCC::GraphApi::Space *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 4 },
        }}),
        // Signal 'spacesAdded'
        QtMocHelpers::SignalData<void(QUuid, QList<OCC::GraphApi::Space*>, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 15, 16 }, { QMetaType::Int, 17 },
        }}),
        // Signal 'spacesRemoved'
        QtMocHelpers::SignalData<void(QUuid, QList<QString>, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QStringList, 19 }, { QMetaType::Int, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SpacesManager, qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::GraphApi::SpacesManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>.metaTypes,
    nullptr
} };

void OCC::GraphApi::SpacesManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpacesManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->spaceChanged((*reinterpret_cast<std::add_pointer_t<Space*>>(_a[1]))); break;
        case 1: _t->updated((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 2: _t->ready(); break;
        case 3: _t->spaceAdded((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::GraphApi::Space*>>(_a[2]))); break;
        case 4: _t->spaceAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::GraphApi::Space*>>(_a[2]))); break;
        case 5: _t->spacesAdded((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<OCC::GraphApi::Space*>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->spacesRemoved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Space* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::GraphApi::Space* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::GraphApi::Space* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::GraphApi::Space*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(Space * ) const>(_a, &SpacesManager::spaceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(Account * )>(_a, &SpacesManager::updated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)() const>(_a, &SpacesManager::ready, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(QUuid , OCC::GraphApi::Space * )>(_a, &SpacesManager::spaceAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(QUuid , OCC::GraphApi::Space * )>(_a, &SpacesManager::spaceAboutToBeRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(QUuid , QList<OCC::GraphApi::Space*> , int )>(_a, &SpacesManager::spacesAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpacesManager::*)(QUuid , QList<QString> , int )>(_a, &SpacesManager::spacesRemoved, 6))
            return;
    }
}

const QMetaObject *OCC::GraphApi::SpacesManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::GraphApi::SpacesManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC8GraphApi13SpacesManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::GraphApi::SpacesManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OCC::GraphApi::SpacesManager::spaceChanged(Space * _t1)const
{
    QMetaObject::activate<void>(const_cast< OCC::GraphApi::SpacesManager *>(this), &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::GraphApi::SpacesManager::updated(Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::GraphApi::SpacesManager::ready()const
{
    QMetaObject::activate(const_cast< OCC::GraphApi::SpacesManager *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::GraphApi::SpacesManager::spaceAdded(QUuid _t1, OCC::GraphApi::Space * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::GraphApi::SpacesManager::spaceAboutToBeRemoved(QUuid _t1, OCC::GraphApi::Space * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void OCC::GraphApi::SpacesManager::spacesAdded(QUuid _t1, QList<OCC::GraphApi::Space*> _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void OCC::GraphApi::SpacesManager::spacesRemoved(QUuid _t1, QList<QString> _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
