/********************************************************************************
** Form generated from reading UI file 'ignorelisteditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IGNORELISTEDITOR_H
#define UI_IGNORELISTEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

namespace OCC {

class Ui_IgnoreListEditor
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QTableWidget *tableWidget;
    QVBoxLayout *verticalLayout;
    QPushButton *addPushButton;
    QPushButton *removePushButton;
    QSpacerItem *verticalSpacer;
    QLabel *descriptionLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OCC__IgnoreListEditor)
    {
        if (OCC__IgnoreListEditor->objectName().isEmpty())
            OCC__IgnoreListEditor->setObjectName("OCC__IgnoreListEditor");
        OCC__IgnoreListEditor->resize(555, 609);
        verticalLayout_2 = new QVBoxLayout(OCC__IgnoreListEditor);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox_2 = new QGroupBox(OCC__IgnoreListEditor);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        tableWidget = new QTableWidget(groupBox_2);
        if (tableWidget->columnCount() < 2)
            tableWidget->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        tableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        tableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        tableWidget->setObjectName("tableWidget");
        tableWidget->setEnabled(true);
        tableWidget->setSelectionMode(QAbstractItemView::SingleSelection);
        tableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
        tableWidget->setColumnCount(2);

        horizontalLayout_2->addWidget(tableWidget);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        addPushButton = new QPushButton(groupBox_2);
        addPushButton->setObjectName("addPushButton");
        addPushButton->setEnabled(true);

        verticalLayout->addWidget(addPushButton);

        removePushButton = new QPushButton(groupBox_2);
        removePushButton->setObjectName("removePushButton");
        removePushButton->setEnabled(true);

        verticalLayout->addWidget(removePushButton);

        verticalSpacer = new QSpacerItem(20, 213, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout_2);

        descriptionLabel = new QLabel(groupBox_2);
        descriptionLabel->setObjectName("descriptionLabel");
        descriptionLabel->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(descriptionLabel->sizePolicy().hasHeightForWidth());
        descriptionLabel->setSizePolicy(sizePolicy);
        descriptionLabel->setTextFormat(Qt::PlainText);
        descriptionLabel->setWordWrap(true);

        verticalLayout_3->addWidget(descriptionLabel);


        verticalLayout_2->addWidget(groupBox_2);

        buttonBox = new QDialogButtonBox(OCC__IgnoreListEditor);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(OCC__IgnoreListEditor);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, OCC__IgnoreListEditor, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, OCC__IgnoreListEditor, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(OCC__IgnoreListEditor);
    } // setupUi

    void retranslateUi(QDialog *OCC__IgnoreListEditor)
    {
        OCC__IgnoreListEditor->setWindowTitle(QCoreApplication::translate("OCC::IgnoreListEditor", "Ignored Files Editor", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("OCC::IgnoreListEditor", "Files Ignored by Patterns", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("OCC::IgnoreListEditor", "Pattern", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("OCC::IgnoreListEditor", "Allow Deletion", nullptr));
        addPushButton->setText(QCoreApplication::translate("OCC::IgnoreListEditor", "Add", nullptr));
        removePushButton->setText(QCoreApplication::translate("OCC::IgnoreListEditor", "Remove", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("OCC::IgnoreListEditor", "Files or folders matching a pattern will not be synchronized. Changes take effect the next time folders are synchronized.\n"
"\n"
"Items where deletion is allowed will be deleted if they prevent a directory from being removed. This is useful for meta data.", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class IgnoreListEditor: public Ui_IgnoreListEditor {};
} // namespace Ui
} // namespace OCC

#endif // UI_IGNORELISTEDITOR_H
