// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>



namespace OCC {

QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_org_ownCloud_resources)

class org_ownCloud_resourcesPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    org_ownCloud_resourcesPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_org_ownCloud_resources)
    }
};

} // namespace OCC

#include "owncloudResourcesplugin_org_ownCloud_resourcesPlugin.moc"
