/********************************************************************************
** Form generated from reading UI file 'aboutdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace OCC {

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer_2;
    QLabel *icon;
    QSpacerItem *verticalSpacer;
    QLabel *aboutText;
    QGroupBox *updaterWidget;
    QHBoxLayout *horizontalLayout_2;
    QLabel *updateStateLabel;
    QPushButton *restartButton;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    QTextBrowser *versionInfo;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OCC__AboutDialog)
    {
        if (OCC__AboutDialog->objectName().isEmpty())
            OCC__AboutDialog->setObjectName("OCC__AboutDialog");
        OCC__AboutDialog->resize(751, 391);
        OCC__AboutDialog->setWindowTitle(QString::fromUtf8("About"));
        verticalLayout = new QVBoxLayout(OCC__AboutDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(OCC__AboutDialog);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        icon = new QLabel(tab);
        icon->setObjectName("icon");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(icon->sizePolicy().hasHeightForWidth());
        icon->setSizePolicy(sizePolicy);
        icon->setMaximumSize(QSize(256, 256));
        icon->setText(QString::fromUtf8(""));
        icon->setPixmap(QPixmap(QString::fromUtf8(":/client/ownCloud/theme/colored/owncloud-icon.svg")));

        verticalLayout_2->addWidget(icon);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);

        aboutText = new QLabel(tab);
        aboutText->setObjectName("aboutText");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(aboutText->sizePolicy().hasHeightForWidth());
        aboutText->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(aboutText);


        verticalLayout_4->addLayout(horizontalLayout);

        updaterWidget = new QGroupBox(tab);
        updaterWidget->setObjectName("updaterWidget");
        horizontalLayout_2 = new QHBoxLayout(updaterWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        updateStateLabel = new QLabel(updaterWidget);
        updateStateLabel->setObjectName("updateStateLabel");

        horizontalLayout_2->addWidget(updateStateLabel);

        restartButton = new QPushButton(updaterWidget);
        restartButton->setObjectName("restartButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(restartButton->sizePolicy().hasHeightForWidth());
        restartButton->setSizePolicy(sizePolicy2);

        horizontalLayout_2->addWidget(restartButton);


        verticalLayout_4->addWidget(updaterWidget);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        versionInfo = new QTextBrowser(tab_2);
        versionInfo->setObjectName("versionInfo");
        versionInfo->setOpenExternalLinks(false);
        versionInfo->setOpenLinks(false);

        verticalLayout_3->addWidget(versionInfo);

        tabWidget->addTab(tab_2, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(OCC__AboutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(tabWidget, versionInfo);
        QWidget::setTabOrder(versionInfo, restartButton);

        retranslateUi(OCC__AboutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, OCC__AboutDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, OCC__AboutDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(OCC__AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *OCC__AboutDialog)
    {
        updaterWidget->setTitle(QString());
        restartButton->setText(QCoreApplication::translate("OCC::AboutDialog", "&Restart && Update", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("OCC::AboutDialog", "About", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("OCC::AboutDialog", "Versions", nullptr));
        (void)OCC__AboutDialog;
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui
} // namespace OCC

#endif // UI_ABOUTDIALOG_H
