/********************************************************************************
** Form generated from reading UI file 'accountview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTVIEW_H
#define UI_ACCOUNTVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "QProgressIndicator.h"
#include "gui/FoldersGui/accountfoldersview.h"

namespace OCC {

class Ui_AccountView
{
public:
    QVBoxLayout *verticalLayout_2;
    QWidget *accountStatus;
    QHBoxLayout *horizontalLayout_4;
    QLabel *warningLabel;
    QLabel *connectionStatusLabel;
    QPushButton *manageAccountButton;
    QStackedWidget *stackedWidget;
    QWidget *loadingPage;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QLabel *label;
    QHBoxLayout *horizontalLayout_6;
    QProgressIndicator *spinner;
    QSpacerItem *verticalSpacer_3;
    OCC::AccountFoldersView *accountFoldersView;

    void setupUi(QWidget *OCC__AccountView)
    {
        if (OCC__AccountView->objectName().isEmpty())
            OCC__AccountView->setObjectName("OCC__AccountView");
        OCC__AccountView->resize(678, 557);
        verticalLayout_2 = new QVBoxLayout(OCC__AccountView);
        verticalLayout_2->setObjectName("verticalLayout_2");
        accountStatus = new QWidget(OCC__AccountView);
        accountStatus->setObjectName("accountStatus");
        horizontalLayout_4 = new QHBoxLayout(accountStatus);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        warningLabel = new QLabel(accountStatus);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(warningLabel->sizePolicy().hasHeightForWidth());
        warningLabel->setSizePolicy(sizePolicy);
        warningLabel->setMinimumSize(QSize(32, 32));
        warningLabel->setMaximumSize(QSize(32, 32));
        warningLabel->setScaledContents(true);

        horizontalLayout_4->addWidget(warningLabel);

        connectionStatusLabel = new QLabel(accountStatus);
        connectionStatusLabel->setObjectName("connectionStatusLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(connectionStatusLabel->sizePolicy().hasHeightForWidth());
        connectionStatusLabel->setSizePolicy(sizePolicy1);
        connectionStatusLabel->setText(QString::fromUtf8("Connection Status"));
        connectionStatusLabel->setTextFormat(Qt::TextFormat::PlainText);
        connectionStatusLabel->setWordWrap(true);

        horizontalLayout_4->addWidget(connectionStatusLabel);

        manageAccountButton = new QPushButton(accountStatus);
        manageAccountButton->setObjectName("manageAccountButton");
        manageAccountButton->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        horizontalLayout_4->addWidget(manageAccountButton);


        verticalLayout_2->addWidget(accountStatus);

        stackedWidget = new QStackedWidget(OCC__AccountView);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(5);
        sizePolicy2.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy2);
        loadingPage = new QWidget();
        loadingPage->setObjectName("loadingPage");
        verticalLayout_4 = new QVBoxLayout(loadingPage);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        label = new QLabel(loadingPage);
        label->setObjectName("label");

        verticalLayout_4->addWidget(label);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        spinner = new QProgressIndicator(loadingPage);
        spinner->setObjectName("spinner");

        horizontalLayout_6->addWidget(spinner);


        verticalLayout_4->addLayout(horizontalLayout_6);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_3);

        stackedWidget->addWidget(loadingPage);
        accountFoldersView = new OCC::AccountFoldersView();
        accountFoldersView->setObjectName("accountFoldersView");
        stackedWidget->addWidget(accountFoldersView);

        verticalLayout_2->addWidget(stackedWidget);


        retranslateUi(OCC__AccountView);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(OCC__AccountView);
    } // setupUi

    void retranslateUi(QWidget *OCC__AccountView)
    {
        OCC__AccountView->setWindowTitle(QCoreApplication::translate("OCC::AccountView", "Form", nullptr));
        warningLabel->setText(QString());
        manageAccountButton->setText(QCoreApplication::translate("OCC::AccountView", "Manage Account", nullptr));
        label->setText(QCoreApplication::translate("OCC::AccountView", "Preparing the account", nullptr));
#if QT_CONFIG(accessibility)
        accountFoldersView->setAccessibleName(QCoreApplication::translate("OCC::AccountView", "Sync connections", nullptr));
#endif // QT_CONFIG(accessibility)
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class AccountView: public Ui_AccountView {};
} // namespace Ui
} // namespace OCC

#endif // UI_ACCOUNTVIEW_H
