/****************************************************************************
** Meta object code from reading C++ file 'connectionvalidator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/connectionvalidator.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connectionvalidator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC19ConnectionValidatorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ConnectionValidator::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ConnectionValidator",
        "connectionResult",
        "",
        "ConnectionValidator::Status",
        "status",
        "errors",
        "sslErrors",
        "QList<QSslError>",
        "checkServer",
        "ConnectionValidator::ValidationMode",
        "mode",
        "systemProxyLookupDone",
        "QNetworkProxy",
        "proxy",
        "checkAuthentication",
        "slotCheckServerAndAuth",
        "slotAuthFailed",
        "slotAuthSuccess",
        "ValidationMode",
        "ValidateServer",
        "ValidateAuth",
        "ValidateAuthAndUpdate",
        "Status",
        "Undefined",
        "Connected",
        "NotConfigured",
        "ServerVersionMismatch",
        "CredentialsNotReady",
        "CredentialsWrong",
        "SslError",
        "StatusNotFound",
        "ServiceUnavailable",
        "MaintenanceMode",
        "Timeout",
        "ClientUnsupported",
        "CaptivePortal"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionResult'
        QtMocHelpers::SignalData<void(ConnectionValidator::Status, const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QStringList, 5 },
        }}),
        // Signal 'sslErrors'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 5 },
        }}),
        // Slot 'checkServer'
        QtMocHelpers::SlotData<void(ConnectionValidator::ValidationMode)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'checkServer'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'systemProxyLookupDone'
        QtMocHelpers::SlotData<void(const QNetworkProxy &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'checkAuthentication'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCheckServerAndAuth'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAuthFailed'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAuthSuccess'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ValidationMode'
        QtMocHelpers::EnumData<enum ValidationMode>(18, 18, QMC::EnumIsScoped).add({
            {   19, ValidationMode::ValidateServer },
            {   20, ValidationMode::ValidateAuth },
            {   21, ValidationMode::ValidateAuthAndUpdate },
        }),
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(22, 22, QMC::EnumFlags{}).add({
            {   23, Status::Undefined },
            {   24, Status::Connected },
            {   25, Status::NotConfigured },
            {   26, Status::ServerVersionMismatch },
            {   27, Status::CredentialsNotReady },
            {   28, Status::CredentialsWrong },
            {   29, Status::SslError },
            {   30, Status::StatusNotFound },
            {   31, Status::ServiceUnavailable },
            {   32, Status::MaintenanceMode },
            {   33, Status::Timeout },
            {   34, Status::ClientUnsupported },
            {   35, Status::CaptivePortal },
        }),
    };
    return QtMocHelpers::metaObjectData<ConnectionValidator, qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ConnectionValidator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.metaTypes,
    nullptr
} };

void OCC::ConnectionValidator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConnectionValidator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionResult((*reinterpret_cast<std::add_pointer_t<ConnectionValidator::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 1: _t->sslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 2: _t->checkServer((*reinterpret_cast<std::add_pointer_t<ConnectionValidator::ValidationMode>>(_a[1]))); break;
        case 3: _t->checkServer(); break;
        case 4: _t->systemProxyLookupDone((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1]))); break;
        case 5: _t->checkAuthentication(); break;
        case 6: _t->slotCheckServerAndAuth(); break;
        case 7: _t->slotAuthFailed(); break;
        case 8: _t->slotAuthSuccess(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConnectionValidator::*)(ConnectionValidator::Status , const QStringList & )>(_a, &ConnectionValidator::connectionResult, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionValidator::*)(const QList<QSslError> & )>(_a, &ConnectionValidator::sslErrors, 1))
            return;
    }
}

const QMetaObject *OCC::ConnectionValidator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ConnectionValidator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ConnectionValidator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OCC::ConnectionValidator::connectionResult(ConnectionValidator::Status _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::ConnectionValidator::sslErrors(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
