/****************************************************************************
** Meta object code from reading C++ file 'syncengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/syncengine.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC10SyncEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncEngine::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10SyncEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncEngine",
        "rootEtag",
        "",
        "aboutToPropagate",
        "SyncFileItemSet",
        "items",
        "itemCompleted",
        "SyncFileItemPtr",
        "transmissionProgress",
        "ProgressInfo",
        "progress",
        "syncError",
        "message",
        "ErrorCategory",
        "category",
        "excluded",
        "path",
        "finished",
        "success",
        "started",
        "seenLockedFile",
        "fileName",
        "FileSystem::LockMode",
        "mode",
        "slotFolderDiscovered",
        "local",
        "folder",
        "slotRootEtagReceived",
        "time",
        "slotItemDiscovered",
        "item",
        "slotNewItem",
        "slotItemCompleted",
        "slotDiscoveryFinished",
        "slotPropagationFinished",
        "slotProgress",
        "SyncFileItem",
        "curent",
        "updateFileTotal",
        "newSize",
        "slotSummaryError",
        "slotInsufficientLocalStorage",
        "slotInsufficientRemoteStorage",
        "startSync"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rootEtag'
        QtMocHelpers::SignalData<void(const QString &, const QDateTime &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QDateTime, 2 },
        }}),
        // Signal 'aboutToPropagate'
        QtMocHelpers::SignalData<void(const SyncFileItemSet &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'itemCompleted'
        QtMocHelpers::SignalData<void(const SyncFileItemPtr &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'transmissionProgress'
        QtMocHelpers::SignalData<void(const ProgressInfo &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'syncError'
        QtMocHelpers::SignalData<void(const QString &, ErrorCategory)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'syncError'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'excluded'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seenLockedFile'
        QtMocHelpers::SignalData<void(const QString &, FileSystem::LockMode)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotFolderDiscovered'
        QtMocHelpers::SlotData<void(bool, const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 25 }, { QMetaType::QString, 26 },
        }}),
        // Slot 'slotRootEtagReceived'
        QtMocHelpers::SlotData<void(const QString &, const QDateTime &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QDateTime, 28 },
        }}),
        // Slot 'slotItemDiscovered'
        QtMocHelpers::SlotData<void(const SyncFileItemPtr &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 30 },
        }}),
        // Slot 'slotNewItem'
        QtMocHelpers::SlotData<void(const SyncFileItemPtr &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 30 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const SyncFileItemPtr &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 30 },
        }}),
        // Slot 'slotDiscoveryFinished'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPropagationFinished'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'slotProgress'
        QtMocHelpers::SlotData<void(const SyncFileItem &, qint64)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 30 }, { QMetaType::LongLong, 37 },
        }}),
        // Slot 'updateFileTotal'
        QtMocHelpers::SlotData<void(const SyncFileItem &, qint64)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 30 }, { QMetaType::LongLong, 39 },
        }}),
        // Slot 'slotSummaryError'
        QtMocHelpers::SlotData<void(const QString &)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'slotInsufficientLocalStorage'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInsufficientRemoteStorage'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'startSync'
        QtMocHelpers::MethodData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SyncEngine, qt_meta_tag_ZN3OCC10SyncEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.metaTypes,
    nullptr
} };

void OCC::SyncEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rootEtag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 1: _t->aboutToPropagate((*reinterpret_cast<std::add_pointer_t<SyncFileItemSet>>(_a[1]))); break;
        case 2: _t->itemCompleted((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 3: _t->transmissionProgress((*reinterpret_cast<std::add_pointer_t<ProgressInfo>>(_a[1]))); break;
        case 4: _t->syncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ErrorCategory>>(_a[2]))); break;
        case 5: _t->syncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->excluded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->started(); break;
        case 9: _t->seenLockedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<FileSystem::LockMode>>(_a[2]))); break;
        case 10: _t->slotFolderDiscovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->slotRootEtagReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 12: _t->slotItemDiscovered((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 13: _t->slotNewItem((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 14: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 15: _t->slotDiscoveryFinished(); break;
        case 16: _t->slotPropagationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->slotProgress((*reinterpret_cast<std::add_pointer_t<SyncFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 18: _t->updateFileTotal((*reinterpret_cast<std::add_pointer_t<SyncFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 19: _t->slotSummaryError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->slotInsufficientLocalStorage(); break;
        case 21: _t->slotInsufficientRemoteStorage(); break;
        case 22: _t->startSync(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & , const QDateTime & )>(_a, &SyncEngine::rootEtag, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const SyncFileItemSet & )>(_a, &SyncEngine::aboutToPropagate, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const SyncFileItemPtr & )>(_a, &SyncEngine::itemCompleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const ProgressInfo & )>(_a, &SyncEngine::transmissionProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & , ErrorCategory )>(_a, &SyncEngine::syncError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & )>(_a, &SyncEngine::excluded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(bool )>(_a, &SyncEngine::finished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)()>(_a, &SyncEngine::started, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & , FileSystem::LockMode )>(_a, &SyncEngine::seenLockedFile, 9))
            return;
    }
}

const QMetaObject *OCC::SyncEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SyncEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::SyncEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void OCC::SyncEngine::rootEtag(const QString & _t1, const QDateTime & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::SyncEngine::aboutToPropagate(const SyncFileItemSet & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::SyncEngine::itemCompleted(const SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::SyncEngine::transmissionProgress(const ProgressInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::SyncEngine::syncError(const QString & _t1, ErrorCategory _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 6
void OCC::SyncEngine::excluded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void OCC::SyncEngine::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void OCC::SyncEngine::started()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OCC::SyncEngine::seenLockedFile(const QString & _t1, FileSystem::LockMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
