/****************************************************************************
** Meta object code from reading C++ file 'folder.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/folder.h"
#include <QtNetwork/QSslError>
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC6FolderE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Folder::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC6FolderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Folder",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Folders can only be created by the FolderManager",
        "syncStateChange",
        "",
        "syncStarted",
        "syncFinished",
        "SyncResult",
        "result",
        "syncPausedChanged",
        "Folder*",
        "paused",
        "canSyncChanged",
        "spaceChanged",
        "vfsModeChanged",
        "f",
        "Vfs::Mode",
        "newMode",
        "displayNameChanged",
        "imageChanged",
        "progressUpdate",
        "ProgressInfo",
        "progress",
        "watchedFileChangedExternally",
        "path",
        "slotTerminateSync",
        "reason",
        "startSync",
        "slotDiscardDownloadProgress",
        "slotWipeErrorBlacklist",
        "slotWatchedPathsChanged",
        "QSet<QString>",
        "paths",
        "ChangeReason",
        "implicitlyHydrateFile",
        "relativepath",
        "slotNextSyncFullLocalDiscovery",
        "schedulePathForLocalDiscovery",
        "relativePath",
        "reloadExcludes",
        "slotSyncStarted",
        "slotSyncFinished",
        "slotSyncError",
        "message",
        "ErrorCategory",
        "category",
        "slotItemCompleted",
        "SyncFileItemPtr",
        "slotLogPropagationStart",
        "slotFolderConflicts",
        "folder",
        "conflictPaths",
        "warnOnNewExcludedItem",
        "SyncJournalFileRecord",
        "record",
        "QStringView",
        "slotWatcherUnreliable",
        "space",
        "GraphApi::Space*",
        "Other",
        "UnLock"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'syncStateChange'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncStarted'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncFinished'
        QtMocHelpers::SignalData<void(const SyncResult &)>(10, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'syncPausedChanged'
        QtMocHelpers::SignalData<void(Folder *, bool)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 8 }, { QMetaType::Bool, 15 },
        }}),
        // Signal 'canSyncChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spaceChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'vfsModeChanged'
        QtMocHelpers::SignalData<void(Folder *, Vfs::Mode)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'displayNameChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imageChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'progressUpdate'
        QtMocHelpers::SignalData<void(const ProgressInfo &)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'watchedFileChangedExternally'
        QtMocHelpers::SignalData<void(const QString &)>(27, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'slotTerminateSync'
        QtMocHelpers::SlotData<void(const QString &)>(29, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'startSync'
        QtMocHelpers::SlotData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDiscardDownloadProgress'
        QtMocHelpers::SlotData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWipeErrorBlacklist'
        QtMocHelpers::SlotData<int()>(33, 8, QMC::AccessPublic, QMetaType::Int),
        // Slot 'slotWatchedPathsChanged'
        QtMocHelpers::SlotData<void(const QSet<QString> &, enum ChangeReason)>(34, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 36 }, { 0x80000000 | 37, 30 },
        }}),
        // Slot 'implicitlyHydrateFile'
        QtMocHelpers::SlotData<void(const QString &)>(38, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
        // Slot 'slotNextSyncFullLocalDiscovery'
        QtMocHelpers::SlotData<void()>(40, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'schedulePathForLocalDiscovery'
        QtMocHelpers::SlotData<void(const QString &)>(41, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 },
        }}),
        // Slot 'reloadExcludes'
        QtMocHelpers::SlotData<bool()>(43, 8, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotSyncStarted'
        QtMocHelpers::SlotData<void()>(44, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncFinished'
        QtMocHelpers::SlotData<void(bool)>(45, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotSyncError'
        QtMocHelpers::SlotData<void(const QString &, ErrorCategory)>(46, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 47 }, { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotSyncError'
        QtMocHelpers::SlotData<void(const QString &)>(46, 8, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 47 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const SyncFileItemPtr &)>(50, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 51, 8 },
        }}),
        // Slot 'slotLogPropagationStart'
        QtMocHelpers::SlotData<void()>(52, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderConflicts'
        QtMocHelpers::SlotData<void(Folder *, const QStringList &)>(53, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 54 }, { QMetaType::QStringList, 55 },
        }}),
        // Slot 'warnOnNewExcludedItem'
        QtMocHelpers::SlotData<void(const SyncJournalFileRecord &, QStringView)>(56, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 57, 58 }, { 0x80000000 | 59, 28 },
        }}),
        // Slot 'slotWatcherUnreliable'
        QtMocHelpers::SlotData<void(const QString &)>(60, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'space'
        QtMocHelpers::PropertyData<GraphApi::Space*>(61, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChangeReason'
        QtMocHelpers::EnumData<enum ChangeReason>(37, 37, QMC::EnumIsScoped).add({
            {   63, ChangeReason::Other },
            {   64, ChangeReason::UnLock },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Folder, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::Folder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC6FolderE_t>.metaTypes,
    nullptr
} };

void OCC::Folder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Folder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->syncStateChange(); break;
        case 1: _t->syncStarted(); break;
        case 2: _t->syncFinished((*reinterpret_cast<std::add_pointer_t<SyncResult>>(_a[1]))); break;
        case 3: _t->syncPausedChanged((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->canSyncChanged(); break;
        case 5: _t->spaceChanged(); break;
        case 6: _t->vfsModeChanged((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vfs::Mode>>(_a[2]))); break;
        case 7: _t->displayNameChanged(); break;
        case 8: _t->imageChanged(); break;
        case 9: _t->progressUpdate((*reinterpret_cast<std::add_pointer_t<ProgressInfo>>(_a[1]))); break;
        case 10: _t->watchedFileChangedExternally((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotTerminateSync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->startSync(); break;
        case 13: _t->slotDiscardDownloadProgress(); break;
        case 14: { int _r = _t->slotWipeErrorBlacklist();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->slotWatchedPathsChanged((*reinterpret_cast<std::add_pointer_t<QSet<QString>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum ChangeReason>>(_a[2]))); break;
        case 16: _t->implicitlyHydrateFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->slotNextSyncFullLocalDiscovery(); break;
        case 18: _t->schedulePathForLocalDiscovery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: { bool _r = _t->reloadExcludes();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->slotSyncStarted(); break;
        case 21: _t->slotSyncFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->slotSyncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ErrorCategory>>(_a[2]))); break;
        case 23: _t->slotSyncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 25: _t->slotLogPropagationStart(); break;
        case 26: _t->slotFolderConflicts((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 27: _t->warnOnNewExcludedItem((*reinterpret_cast<std::add_pointer_t<SyncJournalFileRecord>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringView>>(_a[2]))); break;
        case 28: _t->slotWatcherUnreliable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QString> >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::syncStateChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::syncStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const SyncResult & )>(_a, &Folder::syncFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(Folder * , bool )>(_a, &Folder::syncPausedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::canSyncChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::spaceChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(Folder * , Vfs::Mode )>(_a, &Folder::vfsModeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::displayNameChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::imageChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const ProgressInfo & )>(_a, &Folder::progressUpdate, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const QString & )>(_a, &Folder::watchedFileChangedExternally, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<GraphApi::Space**>(_v) = _t->space(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Folder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Folder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Folder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::Folder::syncStateChange()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Folder::syncStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Folder::syncFinished(const SyncResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::Folder::syncPausedChanged(Folder * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::Folder::canSyncChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Folder::spaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Folder::vfsModeChanged(Folder * _t1, Vfs::Mode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void OCC::Folder::displayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OCC::Folder::imageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OCC::Folder::progressUpdate(const ProgressInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void OCC::Folder::watchedFileChangedExternally(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
