/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<accountmanager.h>)
#  include <accountmanager.h>
#endif
#if __has_include(<accountstate.h>)
#  include <accountstate.h>
#endif
#if __has_include(<folder.h>)
#  include <folder.h>
#endif
#if __has_include(<qmlutils.h>)
#  include <qmlutils.h>
#endif
#if __has_include(<settingsdialog.h>)
#  include <settingsdialog.h>
#endif
#if __has_include(<spacesbrowser.h>)
#  include <spacesbrowser.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
namespace OCC {
Q_QMLTYPE_EXPORT void qml_register_types_org_ownCloud_gui()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<OCC::AccountManager>("org.ownCloud.gui", 1);
    qmlRegisterTypesAndRevisions<OCC::AccountState>("org.ownCloud.gui", 1);
    qmlRegisterEnum<OCC::AccountState::State>("OCC::AccountState::State");
    qmlRegisterTypesAndRevisions<OCC::Folder>("org.ownCloud.gui", 1);
    qmlRegisterEnum<OCC::Folder::ChangeReason>("OCC::Folder::ChangeReason");
    qmlRegisterTypesAndRevisions<OCC::QmlUtils::OCQuickWidget>("org.ownCloud.gui", 1);
    qmlRegisterTypesAndRevisions<OCC::SettingsDialog>("org.ownCloud.gui", 1);
    qmlRegisterEnum<OCC::SettingsDialog::SettingsPage>("OCC::SettingsDialog::SettingsPage");
    qmlRegisterTypesAndRevisions<OCC::Spaces::SpacesBrowser>("org.ownCloud.gui", 1);
    QMetaType::fromType<QMainWindow *>().id();
    qmlRegisterEnum<QMainWindow::DockOption>("QMainWindow::DockOption");
    qmlRegisterEnum<QMainWindow::DockOptions>("QMainWindow::DockOptions");
    qmlRegisterEnum<QMainWindow::DockOption>("QMainWindow::DockOption");
    QMetaType::fromType<QQuickWidget *>().id();
    qmlRegisterEnum<QQuickWidget::ResizeMode>("QQuickWidget::ResizeMode");
    qmlRegisterEnum<QQuickWidget::Status>("QQuickWidget::Status");
    QMetaType::fromType<QWidget *>().id();
    QT_WARNING_POP
    qmlRegisterModule("org.ownCloud.gui", 1, 0);
}

static const QQmlModuleRegistration orgownCloudguiRegistration("org.ownCloud.gui", qml_register_types_org_ownCloud_gui);
} // namespace OCC
