/****************************************************************************
** Meta object code from reading C++ file 'oauth.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/creds/oauth.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'oauth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC5OAuthE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::OAuth::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC5OAuthE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::OAuth",
        "result",
        "",
        "OAuth::Result",
        "token",
        "refreshToken",
        "authorisationLinkChanged",
        "fetchWellKnownFinished",
        "handleSocketReadyRead",
        "Result",
        "NotSupported",
        "LoggedIn",
        "Error",
        "ErrorInsecureUrl",
        "ErrorIdPUnreachable",
        "TokenEndpointAuthMethods",
        "client_secret_basic",
        "client_secret_post",
        "PromptValuesSupported",
        "none",
        "consent",
        "select_account",
        "login",
        "create"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'result'
        QtMocHelpers::SignalData<void(OAuth::Result, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 1 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(OAuth::Result, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 1 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(OAuth::Result)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 1 },
        }}),
        // Signal 'authorisationLinkChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fetchWellKnownFinished'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSocketReadyRead'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Result'
        QtMocHelpers::EnumData<enum Result>(9, 9, QMC::EnumFlags{}).add({
            {   10, Result::NotSupported },
            {   11, Result::LoggedIn },
            {   12, Result::Error },
            {   13, Result::ErrorInsecureUrl },
            {   14, Result::ErrorIdPUnreachable },
        }),
        // enum 'TokenEndpointAuthMethods'
        QtMocHelpers::EnumData<enum TokenEndpointAuthMethods>(15, 15, QMC::EnumIsScoped).add({
            {   16, TokenEndpointAuthMethods::client_secret_basic },
            {   17, TokenEndpointAuthMethods::client_secret_post },
        }),
        // enum 'PromptValuesSupported'
        QtMocHelpers::EnumData<enum PromptValuesSupported>(18, 18, QMC::EnumIsScoped).add({
            {   19, PromptValuesSupported::none },
            {   20, PromptValuesSupported::consent },
            {   21, PromptValuesSupported::select_account },
            {   22, PromptValuesSupported::login },
            {   23, PromptValuesSupported::create },
        }),
    };
    return QtMocHelpers::metaObjectData<OAuth, qt_meta_tag_ZN3OCC5OAuthE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::OAuth::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5OAuthE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5OAuthE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC5OAuthE_t>.metaTypes,
    nullptr
} };

void OCC::OAuth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OAuth *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->result((*reinterpret_cast<std::add_pointer_t<OAuth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->result((*reinterpret_cast<std::add_pointer_t<OAuth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->result((*reinterpret_cast<std::add_pointer_t<OAuth::Result>>(_a[1]))); break;
        case 3: _t->authorisationLinkChanged(); break;
        case 4: _t->fetchWellKnownFinished(); break;
        case 5: _t->handleSocketReadyRead(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OAuth::*)(OAuth::Result , const QString & , const QString & )>(_a, &OAuth::result, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OAuth::*)()>(_a, &OAuth::authorisationLinkChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OAuth::*)()>(_a, &OAuth::fetchWellKnownFinished, 4))
            return;
    }
}

const QMetaObject *OCC::OAuth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::OAuth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5OAuthE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::OAuth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OCC::OAuth::result(OAuth::Result _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void OCC::OAuth::authorisationLinkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::OAuth::fetchWellKnownFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountBasedOAuth::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountBasedOAuth",
        "refreshError",
        "",
        "QNetworkReply::NetworkError",
        "error",
        "errorString",
        "refreshFinished",
        "accessToken",
        "refreshToken"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'refreshError'
        QtMocHelpers::SignalData<void(QNetworkReply::NetworkError, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'refreshFinished'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountBasedOAuth, qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::AccountBasedOAuth::staticMetaObject = { {
    QMetaObject::SuperData::link<OAuth::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>.metaTypes,
    nullptr
} };

void OCC::AccountBasedOAuth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountBasedOAuth *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->refreshError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->refreshFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountBasedOAuth::*)(QNetworkReply::NetworkError , const QString & )>(_a, &AccountBasedOAuth::refreshError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountBasedOAuth::*)(const QString & , const QString & )>(_a, &AccountBasedOAuth::refreshFinished, 1))
            return;
    }
}

const QMetaObject *OCC::AccountBasedOAuth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountBasedOAuth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17AccountBasedOAuthE_t>.strings))
        return static_cast<void*>(this);
    return OAuth::qt_metacast(_clname);
}

int OCC::AccountBasedOAuth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = OAuth::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountBasedOAuth::refreshError(QNetworkReply::NetworkError _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::AccountBasedOAuth::refreshFinished(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
