/****************************************************************************
** Meta object code from reading C++ file 'syncfilestatus.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/common/syncfilestatus.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncfilestatus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14SyncFileStatusE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncFileStatus::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14SyncFileStatusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncFileStatus",
        "SyncFileStatusTag",
        "StatusNone",
        "StatusSync",
        "StatusWarning",
        "StatusUpToDate",
        "StatusError",
        "StatusExcluded"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SyncFileStatusTag'
        QtMocHelpers::EnumData<enum SyncFileStatusTag>(1, 1, QMC::EnumFlags{}).add({
            {    2, SyncFileStatusTag::StatusNone },
            {    3, SyncFileStatusTag::StatusSync },
            {    4, SyncFileStatusTag::StatusWarning },
            {    5, SyncFileStatusTag::StatusUpToDate },
            {    6, SyncFileStatusTag::StatusError },
            {    7, SyncFileStatusTag::StatusExcluded },
        }),
    };
    return QtMocHelpers::metaObjectData<SyncFileStatus, qt_meta_tag_ZN3OCC14SyncFileStatusE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncFileStatus::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14SyncFileStatusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14SyncFileStatusE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14SyncFileStatusE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
