/********************************************************************************
** Form generated from reading UI file 'syncerrorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SYNCERRORWIDGET_H
#define UI_SYNCERRORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace OCC {

class Ui_SyncErrorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *_filterButton;
    QTableView *_tableView;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *_tooManyIssuesWarning;
    QLabel *_conflictHelp;

    void setupUi(QWidget *OCC__SyncErrorWidget)
    {
        if (OCC__SyncErrorWidget->objectName().isEmpty())
            OCC__SyncErrorWidget->setObjectName("OCC__SyncErrorWidget");
        OCC__SyncErrorWidget->resize(580, 578);
        verticalLayout = new QVBoxLayout(OCC__SyncErrorWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        _filterButton = new QPushButton(OCC__SyncErrorWidget);
        _filterButton->setObjectName("_filterButton");

        horizontalLayout->addWidget(_filterButton);


        verticalLayout->addLayout(horizontalLayout);

        _tableView = new QTableView(OCC__SyncErrorWidget);
        _tableView->setObjectName("_tableView");
        _tableView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        _tableView->setTabKeyNavigation(false);
        _tableView->setAlternatingRowColors(true);
        _tableView->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        _tableView->setShowGrid(false);
        _tableView->setSortingEnabled(true);
        _tableView->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        _tableView->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(_tableView);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        _tooManyIssuesWarning = new QLabel(OCC__SyncErrorWidget);
        _tooManyIssuesWarning->setObjectName("_tooManyIssuesWarning");
        _tooManyIssuesWarning->setWordWrap(true);

        verticalLayout_2->addWidget(_tooManyIssuesWarning);

        _conflictHelp = new QLabel(OCC__SyncErrorWidget);
        _conflictHelp->setObjectName("_conflictHelp");
        _conflictHelp->setWordWrap(true);
        _conflictHelp->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(_conflictHelp);


        horizontalLayout_2->addLayout(verticalLayout_2);

        horizontalLayout_2->setStretch(0, 1);

        verticalLayout->addLayout(horizontalLayout_2);

        QWidget::setTabOrder(_filterButton, _tableView);

        retranslateUi(OCC__SyncErrorWidget);

        QMetaObject::connectSlotsByName(OCC__SyncErrorWidget);
    } // setupUi

    void retranslateUi(QWidget *OCC__SyncErrorWidget)
    {
#if QT_CONFIG(accessibility)
        _tableView->setAccessibleName(QCoreApplication::translate("OCC::SyncErrorWidget", "Issues table", nullptr));
#endif // QT_CONFIG(accessibility)
        _tooManyIssuesWarning->setText(QCoreApplication::translate("OCC::SyncErrorWidget", "There were too many issues. Not all will be visible here.", nullptr));
        _conflictHelp->setText(QCoreApplication::translate("OCC::SyncErrorWidget", "There were conflicts. Check the documentation on how to resolve them.", nullptr));
        (void)OCC__SyncErrorWidget;
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class SyncErrorWidget: public Ui_SyncErrorWidget {};
} // namespace Ui
} // namespace OCC

#endif // UI_SYNCERRORWIDGET_H
