/********************************************************************************
** Form generated from reading UI file 'logbrowser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOGBROWSER_H
#define UI_LOGBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

namespace OCC {

class Ui_LogBrowser
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *warningIcon;
    QLabel *label_5;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QLabel *locationLabel;
    QSpacerItem *horizontalSpacer;
    QCheckBox *enableLoggingButton;
    QCheckBox *httpLogButton;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    QSpinBox *spinBox_numberOflogsToKeep;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_3;
    QPushButton *openFolderButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OCC__LogBrowser)
    {
        if (OCC__LogBrowser->objectName().isEmpty())
            OCC__LogBrowser->setObjectName("OCC__LogBrowser");
        OCC__LogBrowser->resize(600, 399);
        verticalLayout = new QVBoxLayout(OCC__LogBrowser);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        warningIcon = new QLabel(OCC__LogBrowser);
        warningIcon->setObjectName("warningIcon");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(warningIcon->sizePolicy().hasHeightForWidth());
        warningIcon->setSizePolicy(sizePolicy);
        warningIcon->setMinimumSize(QSize(64, 64));
        warningIcon->setMaximumSize(QSize(64, 64));
        warningIcon->setPixmap(QPixmap(QString::fromUtf8(":/client/resources/light/warning.svg")));
        warningIcon->setScaledContents(true);

        horizontalLayout_2->addWidget(warningIcon);

        label_5 = new QLabel(OCC__LogBrowser);
        label_5->setObjectName("label_5");
        label_5->setWordWrap(true);

        horizontalLayout_2->addWidget(label_5);


        verticalLayout->addLayout(horizontalLayout_2);

        label = new QLabel(OCC__LogBrowser);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(OCC__LogBrowser);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        locationLabel = new QLabel(OCC__LogBrowser);
        locationLabel->setObjectName("locationLabel");
        locationLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout->addWidget(locationLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        enableLoggingButton = new QCheckBox(OCC__LogBrowser);
        enableLoggingButton->setObjectName("enableLoggingButton");

        verticalLayout->addWidget(enableLoggingButton);

        httpLogButton = new QCheckBox(OCC__LogBrowser);
        httpLogButton->setObjectName("httpLogButton");

        verticalLayout->addWidget(httpLogButton);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_4 = new QLabel(OCC__LogBrowser);
        label_4->setObjectName("label_4");

        horizontalLayout_3->addWidget(label_4);

        spinBox_numberOflogsToKeep = new QSpinBox(OCC__LogBrowser);
        spinBox_numberOflogsToKeep->setObjectName("spinBox_numberOflogsToKeep");
        spinBox_numberOflogsToKeep->setMinimum(5);

        horizontalLayout_3->addWidget(spinBox_numberOflogsToKeep);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_3);

        label_3 = new QLabel(OCC__LogBrowser);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setWordWrap(true);

        verticalLayout->addWidget(label_3);

        openFolderButton = new QPushButton(OCC__LogBrowser);
        openFolderButton->setObjectName("openFolderButton");

        verticalLayout->addWidget(openFolderButton);

        buttonBox = new QDialogButtonBox(OCC__LogBrowser);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(OCC__LogBrowser);

        QMetaObject::connectSlotsByName(OCC__LogBrowser);
    } // setupUi

    void retranslateUi(QDialog *OCC__LogBrowser)
    {
        OCC__LogBrowser->setWindowTitle(QCoreApplication::translate("OCC::LogBrowser", "Log Output", nullptr));
        warningIcon->setText(QString());
        label_5->setText(QCoreApplication::translate("OCC::LogBrowser", "<html><head/><body><p><b>The logs contain sensitive information which you should not make publicly available</b></span></p></body></html>", nullptr));
        label->setText(QCoreApplication::translate("OCC::LogBrowser", "The client can write debug logs to a temporary folder. These logs are very helpful for diagnosing problems.\n"
"Since log files can get large, the client will start a new one for each sync run and compress older ones.", nullptr));
        label_2->setText(QCoreApplication::translate("OCC::LogBrowser", "If enabled, logs will be written to:", nullptr));
        locationLabel->setText(QCoreApplication::translate("OCC::LogBrowser", "C:/log", nullptr));
        enableLoggingButton->setText(QCoreApplication::translate("OCC::LogBrowser", "Enable logging to temporary folder", nullptr));
        httpLogButton->setText(QCoreApplication::translate("OCC::LogBrowser", "Log Http traffic ", nullptr));
        label_4->setText(QCoreApplication::translate("OCC::LogBrowser", "Log files to keep:", nullptr));
        label_3->setText(QCoreApplication::translate("OCC::LogBrowser", "These settings persist across client restarts.\n"
"Note that using any logging command line options will override the settings.", nullptr));
        openFolderButton->setText(QCoreApplication::translate("OCC::LogBrowser", "Open folder", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class LogBrowser: public Ui_LogBrowser {};
} // namespace Ui
} // namespace OCC

#endif // UI_LOGBROWSER_H
