/****************************************************************************
** Meta object code from reading C++ file 'accountstate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/accountstate.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountstate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12AccountStateE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountState::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12AccountStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountState",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only created by AccountManager",
        "stateChanged",
        "",
        "State",
        "state",
        "isConnectedChanged",
        "isSettingUpChanged",
        "settingUp",
        "slotConnectionValidatorResult",
        "ConnectionValidator::Status",
        "status",
        "errors",
        "slotInvalidCredentials",
        "slotCredentialsFetched",
        "account",
        "Account*",
        "isConnected",
        "AccountState::State",
        "SignedOut",
        "Disconnected",
        "Connected",
        "ServiceUnavailable",
        "MaintenanceMode",
        "NetworkError",
        "ConfigurationError",
        "AskingCredentials",
        "Connecting"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(enum State)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'isConnectedChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isSettingUpChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotConnectionValidatorResult'
        QtMocHelpers::SlotData<void(ConnectionValidator::Status, const QStringList &)>(14, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::QStringList, 17 },
        }}),
        // Slot 'slotInvalidCredentials'
        QtMocHelpers::SlotData<void()>(18, 8, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCredentialsFetched'
        QtMocHelpers::SlotData<void()>(19, 8, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'account'
        QtMocHelpers::PropertyData<Account*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'isConnected'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'state'
        QtMocHelpers::PropertyData<AccountState::State>(10, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(9, 9, QMC::EnumFlags{}).add({
            {   24, State::SignedOut },
            {   25, State::Disconnected },
            {   26, State::Connected },
            {   27, State::ServiceUnavailable },
            {   28, State::MaintenanceMode },
            {   29, State::NetworkError },
            {   30, State::ConfigurationError },
            {   31, State::AskingCredentials },
            {   32, State::Connecting },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<AccountState, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::AccountState::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.metaTypes,
    nullptr
} };

void OCC::AccountState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountState *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<enum State>>(_a[1]))); break;
        case 1: _t->isConnectedChanged(); break;
        case 2: _t->isSettingUpChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotConnectionValidatorResult((*reinterpret_cast<std::add_pointer_t<ConnectionValidator::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 4: _t->slotInvalidCredentials(); break;
        case 5: _t->slotCredentialsFetched(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)(State )>(_a, &AccountState::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)()>(_a, &AccountState::isConnectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)(bool )>(_a, &AccountState::isSettingUpChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Account* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Account**>(_v) = _t->accountForQml(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isConnected(); break;
        case 2: *reinterpret_cast<AccountState::State*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::AccountState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::AccountState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountState::stateChanged(State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::AccountState::isConnectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::AccountState::isSettingUpChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
