/********************************************************************************
** Form generated from reading UI file 'spacesbrowser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPACESBROWSER_H
#define UI_SPACESBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "gui/qmlutils.h"

QT_BEGIN_NAMESPACE

class Ui_SpacesBrowser
{
public:
    QVBoxLayout *verticalLayout;
    OCC::QmlUtils::OCQuickWidget *quickWidget;

    void setupUi(QWidget *SpacesBrowser)
    {
        if (SpacesBrowser->objectName().isEmpty())
            SpacesBrowser->setObjectName("SpacesBrowser");
        SpacesBrowser->resize(400, 300);
        verticalLayout = new QVBoxLayout(SpacesBrowser);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        quickWidget = new OCC::QmlUtils::OCQuickWidget(SpacesBrowser);
        quickWidget->setObjectName("quickWidget");
        quickWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        verticalLayout->addWidget(quickWidget);


        retranslateUi(SpacesBrowser);

        QMetaObject::connectSlotsByName(SpacesBrowser);
    } // setupUi

    void retranslateUi(QWidget *SpacesBrowser)
    {
        SpacesBrowser->setWindowTitle(QCoreApplication::translate("SpacesBrowser", "Form", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SpacesBrowser: public Ui_SpacesBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SPACESBROWSER_H
