/****************************************************************************
** Meta object code from reading C++ file 'socketapi.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/socketapi/socketapi.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'socketapi.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9SocketApiE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SocketApi::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9SocketApiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SocketApi",
        "shareCommandReceived",
        "",
        "sharePath",
        "localPath",
        "registerAccount",
        "Account*",
        "a",
        "unregisterAccount",
        "AccountState*",
        "state",
        "slotUpdateFolderView",
        "Folder*",
        "f",
        "slotUnregisterPath",
        "slotRegisterPath",
        "broadcastStatusPushMessage",
        "systemPath",
        "SyncFileStatus",
        "fileStatus",
        "slotNewConnection",
        "slotReadSocket",
        "copyUrlToClipboard",
        "QUrl",
        "link",
        "emailPrivateLink",
        "openPrivateLink",
        "command_RETRIEVE_FOLDER_STATUS",
        "argument",
        "SocketListener*",
        "listener",
        "command_RETRIEVE_FILE_STATUS",
        "command_VERSION",
        "command_SHARE_MENU_TITLE",
        "command_SHARE",
        "localFile",
        "command_COPY_PRIVATE_LINK",
        "command_EMAIL_PRIVATE_LINK",
        "command_OPEN_PRIVATE_LINK",
        "command_OPEN_PRIVATE_LINK_VERSIONS",
        "command_MAKE_AVAILABLE_LOCALLY",
        "filesArg",
        "command_MAKE_ONLINE_ONLY",
        "command_DELETE_ITEM",
        "command_MOVE_ITEM",
        "command_OPEN_APP_LINK",
        "command_V2_GET_CLIENT_ICON",
        "QSharedPointer<SocketApiJobV2>",
        "job",
        "command_GET_STRINGS",
        "command_GET_MENU_ITEMS"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shareCommandReceived'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'registerAccount'
        QtMocHelpers::SlotData<void(Account *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'unregisterAccount'
        QtMocHelpers::SlotData<void(AccountState *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotUpdateFolderView'
        QtMocHelpers::SlotData<void(Folder *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotUnregisterPath'
        QtMocHelpers::SlotData<void(Folder *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotRegisterPath'
        QtMocHelpers::SlotData<void(Folder *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'broadcastStatusPushMessage'
        QtMocHelpers::SlotData<void(const QString &, SyncFileStatus)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotNewConnection'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotReadSocket'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'copyUrlToClipboard'
        QtMocHelpers::SlotData<void(const QUrl &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'emailPrivateLink'
        QtMocHelpers::SlotData<void(const QUrl &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'openPrivateLink'
        QtMocHelpers::SlotData<void(const QUrl &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Method 'command_RETRIEVE_FOLDER_STATUS'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_RETRIEVE_FILE_STATUS'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_VERSION'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_SHARE_MENU_TITLE'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_SHARE'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_COPY_PRIVATE_LINK'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_EMAIL_PRIVATE_LINK'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_OPEN_PRIVATE_LINK'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_OPEN_PRIVATE_LINK_VERSIONS'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_MAKE_AVAILABLE_LOCALLY'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 41 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_MAKE_ONLINE_ONLY'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 41 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_DELETE_ITEM'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_MOVE_ITEM'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_OPEN_APP_LINK'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_V2_GET_CLIENT_ICON'
        QtMocHelpers::MethodData<void(const QSharedPointer<SocketApiJobV2> &) const>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Method 'command_GET_STRINGS'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Method 'command_GET_MENU_ITEMS'
        QtMocHelpers::MethodData<void(const QString &, SocketListener *)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SocketApi, qt_meta_tag_ZN3OCC9SocketApiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SocketApi::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9SocketApiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9SocketApiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9SocketApiE_t>.metaTypes,
    nullptr
} };

void OCC::SocketApi::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SocketApi *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shareCommandReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->registerAccount((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 2: _t->unregisterAccount((*reinterpret_cast<std::add_pointer_t<AccountState*>>(_a[1]))); break;
        case 3: _t->slotUpdateFolderView((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 4: _t->slotUnregisterPath((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 5: _t->slotRegisterPath((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 6: _t->broadcastStatusPushMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SyncFileStatus>>(_a[2]))); break;
        case 7: _t->slotNewConnection(); break;
        case 8: _t->slotReadSocket(); break;
        case 9: _t->copyUrlToClipboard((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 10: _t->emailPrivateLink((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 11: _t->openPrivateLink((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 12: _t->command_RETRIEVE_FOLDER_STATUS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 13: _t->command_RETRIEVE_FILE_STATUS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 14: _t->command_VERSION((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 15: _t->command_SHARE_MENU_TITLE((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 16: _t->command_SHARE((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 17: _t->command_COPY_PRIVATE_LINK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 18: _t->command_EMAIL_PRIVATE_LINK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 19: _t->command_OPEN_PRIVATE_LINK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 20: _t->command_OPEN_PRIVATE_LINK_VERSIONS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 21: _t->command_MAKE_AVAILABLE_LOCALLY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 22: _t->command_MAKE_ONLINE_ONLY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 23: _t->command_DELETE_ITEM((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 24: _t->command_MOVE_ITEM((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 25: _t->command_OPEN_APP_LINK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 26: _t->command_V2_GET_CLIENT_ICON((*reinterpret_cast<std::add_pointer_t<QSharedPointer<SocketApiJobV2>>>(_a[1]))); break;
        case 27: _t->command_GET_STRINGS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        case 28: _t->command_GET_MENU_ITEMS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SocketListener*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SocketApi::*)(const QString & , const QString & )>(_a, &SocketApi::shareCommandReceived, 0))
            return;
    }
}

const QMetaObject *OCC::SocketApi::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SocketApi::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9SocketApiE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::SocketApi::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void OCC::SocketApi::shareCommandReceived(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
