/*
 * Copyright (C) Lisa Reese <lisa.reese@kiteworks.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 */
#pragma once

#include <QObject>

namespace OCC::NewAccount {
Q_NAMESPACE

// I am not making this a strongly typed enum class as it's used to support the radio button ids in the advanced settings gui.
// an old fashioned enum does not require as many casts back and forth, which for this use case is a good thing.
enum SyncType { NONE, USE_VFS, SYNC_ALL, SELECTIVE_SYNC };

Q_ENUM_NS(SyncType)

}
