/********************************************************************************
** Form generated from reading UI file 'spacespage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPACESPAGE_H
#define UI_SPACESPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include "gui/spaces/spacesbrowser.h"

QT_BEGIN_NAMESPACE

class Ui_SpacesPage
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    OCC::Spaces::SpacesBrowser *widget;

    void setupUi(QWizardPage *SpacesPage)
    {
        if (SpacesPage->objectName().isEmpty())
            SpacesPage->setObjectName("SpacesPage");
        SpacesPage->resize(362, 300);
        verticalLayout = new QVBoxLayout(SpacesPage);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(SpacesPage);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("placeholder text"));
        label->setTextFormat(Qt::TextFormat::PlainText);

        verticalLayout->addWidget(label);

        widget = new OCC::Spaces::SpacesBrowser(SpacesPage);
        widget->setObjectName("widget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(widget->sizePolicy().hasHeightForWidth());
        widget->setSizePolicy(sizePolicy);
        widget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        verticalLayout->addWidget(widget);


        retranslateUi(SpacesPage);

        QMetaObject::connectSlotsByName(SpacesPage);
    } // setupUi

    void retranslateUi(QWizardPage *SpacesPage)
    {
#if QT_CONFIG(accessibility)
        widget->setAccessibleName(QCoreApplication::translate("SpacesPage", "Spaces list", nullptr));
#endif // QT_CONFIG(accessibility)
        (void)SpacesPage;
    } // retranslateUi

};

namespace Ui {
    class SpacesPage: public Ui_SpacesPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SPACESPAGE_H
