/****************************************************************************
** Meta object code from reading C++ file 'owncloudpropagator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/owncloudpropagator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'owncloudpropagator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC13PropagatorJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagatorJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC13PropagatorJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagatorJob",
        "finished",
        "",
        "SyncFileItem::Status",
        "abortFinished",
        "status",
        "abort",
        "PropagatorJob::AbortType",
        "abortType",
        "scheduleSelfOrChild",
        "AbortType",
        "Synchronous",
        "Asynchronous",
        "JobState",
        "NotYetStarted",
        "Running",
        "Finished",
        "JobParallelism",
        "FullParallelism",
        "WaitForFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void(SyncFileItem::Status)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'abortFinished'
        QtMocHelpers::SignalData<void(SyncFileItem::Status)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 5 },
        }}),
        // Signal 'abortFinished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'abort'
        QtMocHelpers::SlotData<void(PropagatorJob::AbortType)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'scheduleSelfOrChild'
        QtMocHelpers::SlotData<bool()>(9, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AbortType'
        QtMocHelpers::EnumData<enum AbortType>(10, 10, QMC::EnumFlags{}).add({
            {   11, AbortType::Synchronous },
            {   12, AbortType::Asynchronous },
        }),
        // enum 'JobState'
        QtMocHelpers::EnumData<enum JobState>(13, 13, QMC::EnumFlags{}).add({
            {   14, JobState::NotYetStarted },
            {   15, JobState::Running },
            {   16, JobState::Finished },
        }),
        // enum 'JobParallelism'
        QtMocHelpers::EnumData<enum JobParallelism>(17, 17, QMC::EnumFlags{}).add({
            {   18, JobParallelism::FullParallelism },
            {   19, JobParallelism::WaitForFinished },
        }),
    };
    return QtMocHelpers::metaObjectData<PropagatorJob, qt_meta_tag_ZN3OCC13PropagatorJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagatorJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC13PropagatorJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC13PropagatorJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC13PropagatorJobE_t>.metaTypes,
    nullptr
} };

void OCC::PropagatorJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagatorJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 1: _t->abortFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 2: _t->abortFinished(); break;
        case 3: _t->abort((*reinterpret_cast<std::add_pointer_t<PropagatorJob::AbortType>>(_a[1]))); break;
        case 4: { bool _r = _t->scheduleSelfOrChild();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PropagatorJob::*)(SyncFileItem::Status )>(_a, &PropagatorJob::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PropagatorJob::*)(SyncFileItem::Status )>(_a, &PropagatorJob::abortFinished, 1))
            return;
    }
}

const QMetaObject *OCC::PropagatorJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagatorJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC13PropagatorJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::PropagatorJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::PropagatorJob::finished(SyncFileItem::Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::PropagatorJob::abortFinished(SyncFileItem::Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC16PropagateItemJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateItemJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC16PropagateItemJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateItemJob",
        "start",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateItemJob, qt_meta_tag_ZN3OCC16PropagateItemJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateItemJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagatorJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16PropagateItemJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16PropagateItemJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC16PropagateItemJobE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateItemJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateItemJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->start(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *OCC::PropagateItemJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateItemJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16PropagateItemJobE_t>.strings))
        return static_cast<void*>(this);
    return PropagatorJob::qt_metacast(_clname);
}

int OCC::PropagateItemJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagatorJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagatorCompositeJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagatorCompositeJob",
        "slotSubJobAbortFinished",
        "",
        "possiblyRunNextJob",
        "PropagatorJob*",
        "next",
        "slotSubJobFinished",
        "SyncFileItem::Status",
        "status",
        "finalize"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSubJobAbortFinished'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'possiblyRunNextJob'
        QtMocHelpers::SlotData<bool(PropagatorJob *)>(3, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotSubJobFinished'
        QtMocHelpers::SlotData<void(SyncFileItem::Status)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'finalize'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagatorCompositeJob, qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagatorCompositeJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagatorJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>.metaTypes,
    nullptr
} };

void OCC::PropagatorCompositeJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagatorCompositeJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSubJobAbortFinished(); break;
        case 1: { bool _r = _t->possiblyRunNextJob((*reinterpret_cast<std::add_pointer_t<PropagatorJob*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->slotSubJobFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 3: _t->finalize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PropagatorJob* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OCC::PropagatorCompositeJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagatorCompositeJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagatorCompositeJobE_t>.strings))
        return static_cast<void*>(this);
    return PropagatorJob::qt_metacast(_clname);
}

int OCC::PropagatorCompositeJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagatorJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC18PropagateDirectoryE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateDirectory::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateDirectory",
        "start",
        "",
        "slotFirstJobFinished",
        "SyncFileItem::Status",
        "status",
        "slotSubJobsFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFirstJobFinished'
        QtMocHelpers::SlotData<void(SyncFileItem::Status)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotSubJobsFinished'
        QtMocHelpers::SlotData<void(const SyncFileItem::Status)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateDirectory, qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateDirectory::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateItemJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateDirectory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateDirectory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->start(); break;
        case 1: _t->slotFirstJobFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 2: _t->slotSubJobsFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::PropagateDirectory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateDirectory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateDirectoryE_t>.strings))
        return static_cast<void*>(this);
    return PropagateItemJob::qt_metacast(_clname);
}

int OCC::PropagateDirectory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateItemJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateRootDirectory::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateRootDirectory",
        "slotSubJobsFinished",
        "",
        "SyncFileItem::Status",
        "status",
        "slotDirDeletionJobsFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSubJobsFinished'
        QtMocHelpers::SlotData<void(SyncFileItem::Status)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotDirDeletionJobsFinished'
        QtMocHelpers::SlotData<void(SyncFileItem::Status)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateRootDirectory, qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateRootDirectory::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateDirectory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateRootDirectory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateRootDirectory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSubJobsFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 1: _t->slotDirDeletionJobsFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::PropagateRootDirectory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateRootDirectory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22PropagateRootDirectoryE_t>.strings))
        return static_cast<void*>(this);
    return PropagateDirectory::qt_metacast(_clname);
}

int OCC::PropagateRootDirectory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateDirectory::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateIgnoreJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateIgnoreJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateIgnoreJob, qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateIgnoreJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateItemJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateIgnoreJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateIgnoreJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OCC::PropagateIgnoreJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateIgnoreJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18PropagateIgnoreJobE_t>.strings))
        return static_cast<void*>(this);
    return PropagateItemJob::qt_metacast(_clname);
}

int OCC::PropagateIgnoreJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateItemJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::OwncloudPropagator::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::OwncloudPropagator",
        "newItem",
        "",
        "SyncFileItemPtr",
        "itemCompleted",
        "progress",
        "SyncFileItem",
        "bytes",
        "updateFileTotal",
        "newSize",
        "finished",
        "success",
        "seenLockedFile",
        "fileName",
        "FileSystem::LockMode",
        "mode",
        "insufficientLocalStorage",
        "insufficientRemoteStorage",
        "abortTimeout",
        "emitFinished",
        "SyncFileItem::Status",
        "status",
        "scheduleNextJobImpl",
        "DiskSpaceResult",
        "DiskSpaceOk",
        "DiskSpaceFailure",
        "DiskSpaceCritical"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newItem'
        QtMocHelpers::SignalData<void(const SyncFileItemPtr &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'itemCompleted'
        QtMocHelpers::SignalData<void(const SyncFileItemPtr &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(const SyncFileItem &, qint64)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { QMetaType::LongLong, 7 },
        }}),
        // Signal 'updateFileTotal'
        QtMocHelpers::SignalData<void(const SyncFileItem &, qint64)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { QMetaType::LongLong, 9 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'seenLockedFile'
        QtMocHelpers::SignalData<void(const QString &, FileSystem::LockMode)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'insufficientLocalStorage'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'insufficientRemoteStorage'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'abortTimeout'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'emitFinished'
        QtMocHelpers::SlotData<void(SyncFileItem::Status)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'scheduleNextJobImpl'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DiskSpaceResult'
        QtMocHelpers::EnumData<enum DiskSpaceResult>(23, 23, QMC::EnumFlags{}).add({
            {   24, DiskSpaceResult::DiskSpaceOk },
            {   25, DiskSpaceResult::DiskSpaceFailure },
            {   26, DiskSpaceResult::DiskSpaceCritical },
        }),
    };
    return QtMocHelpers::metaObjectData<OwncloudPropagator, qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::OwncloudPropagator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>.metaTypes,
    nullptr
} };

void OCC::OwncloudPropagator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OwncloudPropagator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newItem((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 1: _t->itemCompleted((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 2: _t->progress((*reinterpret_cast<std::add_pointer_t<SyncFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 3: _t->updateFileTotal((*reinterpret_cast<std::add_pointer_t<SyncFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 4: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->seenLockedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<FileSystem::LockMode>>(_a[2]))); break;
        case 6: _t->insufficientLocalStorage(); break;
        case 7: _t->insufficientRemoteStorage(); break;
        case 8: _t->abortTimeout(); break;
        case 9: _t->emitFinished((*reinterpret_cast<std::add_pointer_t<SyncFileItem::Status>>(_a[1]))); break;
        case 10: _t->scheduleNextJobImpl(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(const SyncFileItemPtr & )>(_a, &OwncloudPropagator::newItem, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(const SyncFileItemPtr & )>(_a, &OwncloudPropagator::itemCompleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(const SyncFileItem & , qint64 )>(_a, &OwncloudPropagator::progress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(const SyncFileItem & , qint64 )>(_a, &OwncloudPropagator::updateFileTotal, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(bool )>(_a, &OwncloudPropagator::finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)(const QString & , FileSystem::LockMode )>(_a, &OwncloudPropagator::seenLockedFile, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)()>(_a, &OwncloudPropagator::insufficientLocalStorage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudPropagator::*)()>(_a, &OwncloudPropagator::insufficientRemoteStorage, 7))
            return;
    }
}

const QMetaObject *OCC::OwncloudPropagator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::OwncloudPropagator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18OwncloudPropagatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::OwncloudPropagator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OCC::OwncloudPropagator::newItem(const SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::OwncloudPropagator::itemCompleted(const SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::OwncloudPropagator::progress(const SyncFileItem & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::OwncloudPropagator::updateFileTotal(const SyncFileItem & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::OwncloudPropagator::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::OwncloudPropagator::seenLockedFile(const QString & _t1, FileSystem::LockMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void OCC::OwncloudPropagator::insufficientLocalStorage()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::OwncloudPropagator::insufficientRemoteStorage()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateUpdateMetaDataJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateUpdateMetaDataJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateUpdateMetaDataJob, qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateUpdateMetaDataJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateItemJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateUpdateMetaDataJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateUpdateMetaDataJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OCC::PropagateUpdateMetaDataJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateUpdateMetaDataJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC26PropagateUpdateMetaDataJobE_t>.strings))
        return static_cast<void*>(this);
    return PropagateItemJob::qt_metacast(_clname);
}

int OCC::PropagateUpdateMetaDataJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateItemJob::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
