/****************************************************************************
** Meta object code from reading C++ file 'checksumalgorithms.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/common/checksumalgorithms.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'checksumalgorithms.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9CheckSumsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::CheckSums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9CheckSumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::CheckSums",
        "Algorithm",
        "NONE",
        "SHA3_256",
        "SHA256",
        "SHA1",
        "MD5",
        "ADLER32",
        "DUMMY_FOR_TESTS",
        "PARSE_ERROR"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Algorithm'
        QtMocHelpers::EnumData<Algorithm>(1, 1, QMC::EnumIsScoped).add({
            {    2, Algorithm::NONE },
            {    3, Algorithm::SHA3_256 },
            {    4, Algorithm::SHA256 },
            {    5, Algorithm::SHA1 },
            {    6, Algorithm::MD5 },
            {    7, Algorithm::ADLER32 },
            {    8, Algorithm::DUMMY_FOR_TESTS },
            {    9, Algorithm::PARSE_ERROR },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCC9CheckSumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCC9CheckSumsE =
    OCC::CheckSums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9CheckSumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCC9CheckSumsE =
    qt_staticMetaObjectContent_ZN3OCC9CheckSumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCC9CheckSumsE =
    qt_staticMetaObjectContent_ZN3OCC9CheckSumsE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::CheckSums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCC9CheckSumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCC9CheckSumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCC9CheckSumsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
