/********************************************************************************
** Form generated from reading UI file 'networksettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKSETTINGS_H
#define UI_NETWORKSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace OCC {

class Ui_NetworkSettings
{
public:
    QVBoxLayout *verticalLayout_5;
    QCheckBox *pauseSyncWhenMeteredCheckbox;
    QGroupBox *proxyGroupBox;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *noProxyRadioButton;
    QRadioButton *systemProxyRadioButton;
    QRadioButton *manualProxyRadioButton;
    QWidget *manualSettings;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QComboBox *typeComboBox;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout;
    QLabel *hostLabel;
    QLineEdit *hostLineEdit;
    QLabel *label;
    QSpinBox *portSpinBox;
    QCheckBox *authRequiredcheckBox;
    QWidget *authWidgets;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *userLineEdit;
    QLineEdit *passwordLineEdit;
    QLabel *labelLocalhost;
    QSpacerItem *verticalSpacer;
    QButtonGroup *proxyButtonGroup;

    void setupUi(QWidget *OCC__NetworkSettings)
    {
        if (OCC__NetworkSettings->objectName().isEmpty())
            OCC__NetworkSettings->setObjectName("OCC__NetworkSettings");
        OCC__NetworkSettings->resize(623, 360);
        verticalLayout_5 = new QVBoxLayout(OCC__NetworkSettings);
        verticalLayout_5->setObjectName("verticalLayout_5");
        pauseSyncWhenMeteredCheckbox = new QCheckBox(OCC__NetworkSettings);
        pauseSyncWhenMeteredCheckbox->setObjectName("pauseSyncWhenMeteredCheckbox");

        verticalLayout_5->addWidget(pauseSyncWhenMeteredCheckbox);

        proxyGroupBox = new QGroupBox(OCC__NetworkSettings);
        proxyGroupBox->setObjectName("proxyGroupBox");
        proxyGroupBox->setEnabled(true);
        verticalLayout_4 = new QVBoxLayout(proxyGroupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        noProxyRadioButton = new QRadioButton(proxyGroupBox);
        proxyButtonGroup = new QButtonGroup(OCC__NetworkSettings);
        proxyButtonGroup->setObjectName("proxyButtonGroup");
        proxyButtonGroup->addButton(noProxyRadioButton);
        noProxyRadioButton->setObjectName("noProxyRadioButton");
        noProxyRadioButton->setChecked(true);

        verticalLayout_4->addWidget(noProxyRadioButton);

        systemProxyRadioButton = new QRadioButton(proxyGroupBox);
        proxyButtonGroup->addButton(systemProxyRadioButton);
        systemProxyRadioButton->setObjectName("systemProxyRadioButton");

        verticalLayout_4->addWidget(systemProxyRadioButton);

        manualProxyRadioButton = new QRadioButton(proxyGroupBox);
        proxyButtonGroup->addButton(manualProxyRadioButton);
        manualProxyRadioButton->setObjectName("manualProxyRadioButton");

        verticalLayout_4->addWidget(manualProxyRadioButton);

        manualSettings = new QWidget(proxyGroupBox);
        manualSettings->setObjectName("manualSettings");
        manualSettings->setEnabled(false);
        verticalLayout = new QVBoxLayout(manualSettings);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(-1, -1, 0, -1);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(manualSettings);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        typeComboBox = new QComboBox(manualSettings);
        typeComboBox->setObjectName("typeComboBox");
        typeComboBox->setEnabled(false);

        horizontalLayout_3->addWidget(typeComboBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        hostLabel = new QLabel(manualSettings);
        hostLabel->setObjectName("hostLabel");

        horizontalLayout->addWidget(hostLabel);

        hostLineEdit = new QLineEdit(manualSettings);
        hostLineEdit->setObjectName("hostLineEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(hostLineEdit->sizePolicy().hasHeightForWidth());
        hostLineEdit->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(hostLineEdit);

        label = new QLabel(manualSettings);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        portSpinBox = new QSpinBox(manualSettings);
        portSpinBox->setObjectName("portSpinBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(portSpinBox->sizePolicy().hasHeightForWidth());
        portSpinBox->setSizePolicy(sizePolicy1);
        portSpinBox->setMinimum(1);
        portSpinBox->setMaximum(65535);
        portSpinBox->setValue(8080);

        horizontalLayout->addWidget(portSpinBox);


        verticalLayout->addLayout(horizontalLayout);

        authRequiredcheckBox = new QCheckBox(manualSettings);
        authRequiredcheckBox->setObjectName("authRequiredcheckBox");
        authRequiredcheckBox->setEnabled(false);

        verticalLayout->addWidget(authRequiredcheckBox);

        authWidgets = new QWidget(manualSettings);
        authWidgets->setObjectName("authWidgets");
        authWidgets->setEnabled(false);
        horizontalLayout_2 = new QHBoxLayout(authWidgets);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        userLineEdit = new QLineEdit(authWidgets);
        userLineEdit->setObjectName("userLineEdit");

        horizontalLayout_2->addWidget(userLineEdit);

        passwordLineEdit = new QLineEdit(authWidgets);
        passwordLineEdit->setObjectName("passwordLineEdit");
        passwordLineEdit->setEchoMode(QLineEdit::EchoMode::Password);

        horizontalLayout_2->addWidget(passwordLineEdit);


        verticalLayout->addWidget(authWidgets);

        labelLocalhost = new QLabel(manualSettings);
        labelLocalhost->setObjectName("labelLocalhost");

        verticalLayout->addWidget(labelLocalhost);


        verticalLayout_4->addWidget(manualSettings);


        verticalLayout_5->addWidget(proxyGroupBox);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(typeComboBox);
        hostLabel->setBuddy(hostLineEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(pauseSyncWhenMeteredCheckbox, noProxyRadioButton);
        QWidget::setTabOrder(noProxyRadioButton, systemProxyRadioButton);
        QWidget::setTabOrder(systemProxyRadioButton, manualProxyRadioButton);
        QWidget::setTabOrder(manualProxyRadioButton, typeComboBox);
        QWidget::setTabOrder(typeComboBox, hostLineEdit);
        QWidget::setTabOrder(hostLineEdit, portSpinBox);
        QWidget::setTabOrder(portSpinBox, authRequiredcheckBox);
        QWidget::setTabOrder(authRequiredcheckBox, userLineEdit);
        QWidget::setTabOrder(userLineEdit, passwordLineEdit);

        retranslateUi(OCC__NetworkSettings);

        QMetaObject::connectSlotsByName(OCC__NetworkSettings);
    } // setupUi

    void retranslateUi(QWidget *OCC__NetworkSettings)
    {
        OCC__NetworkSettings->setWindowTitle(QCoreApplication::translate("OCC::NetworkSettings", "Form", nullptr));
        pauseSyncWhenMeteredCheckbox->setText(QCoreApplication::translate("OCC::NetworkSettings", "Pause synchronization when the Internet connection is metered", nullptr));
        proxyGroupBox->setTitle(QCoreApplication::translate("OCC::NetworkSettings", "Proxy Settings", nullptr));
        noProxyRadioButton->setText(QCoreApplication::translate("OCC::NetworkSettings", "No Proxy", nullptr));
        systemProxyRadioButton->setText(QCoreApplication::translate("OCC::NetworkSettings", "Use system proxy", nullptr));
        manualProxyRadioButton->setText(QCoreApplication::translate("OCC::NetworkSettings", "Specify proxy manually", nullptr));
        label_2->setText(QCoreApplication::translate("OCC::NetworkSettings", "Proxy type", nullptr));
        hostLabel->setText(QCoreApplication::translate("OCC::NetworkSettings", "Host", nullptr));
#if QT_CONFIG(accessibility)
        hostLineEdit->setAccessibleName(QCoreApplication::translate("OCC::NetworkSettings", "Proxy Hostname", nullptr));
#endif // QT_CONFIG(accessibility)
        hostLineEdit->setText(QString());
        label->setText(QCoreApplication::translate("OCC::NetworkSettings", ":", nullptr));
#if QT_CONFIG(accessibility)
        portSpinBox->setAccessibleName(QCoreApplication::translate("OCC::NetworkSettings", "Proxy Port Number", nullptr));
#endif // QT_CONFIG(accessibility)
        authRequiredcheckBox->setText(QCoreApplication::translate("OCC::NetworkSettings", "Proxy server requires authentication", nullptr));
        userLineEdit->setText(QString());
        passwordLineEdit->setText(QString());
        labelLocalhost->setText(QCoreApplication::translate("OCC::NetworkSettings", "Note: proxy settings have no effects for accounts on localhost", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class NetworkSettings: public Ui_NetworkSettings {};
} // namespace Ui
} // namespace OCC

#endif // UI_NETWORKSETTINGS_H
