/****************************************************************************
** Meta object code from reading C++ file 'folderman.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/folderman.h"
#include <QtNetwork/QSslError>
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folderman.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9FolderManE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderMan::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9FolderManE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderMan",
        "folderSyncStateChange",
        "",
        "Folder*",
        "folderListChanged",
        "QUuid",
        "accountId",
        "QList<Folder*>",
        "folders",
        "folderAdded",
        "folder",
        "folderRemoved",
        "unsyncedSpaceCountChanged",
        "unsyncedCount",
        "totalSpaces",
        "slotIsConnectedChanged",
        "slotSyncOnceFileUnlocks",
        "path",
        "FileSystem::LockMode",
        "mode",
        "slotReloadSyncOptions",
        "removeFolderFromGui",
        "f",
        "forceFolderSync",
        "slotFolderSyncPauseChanged",
        "paused",
        "slotFolderCanSyncChanged",
        "slotFolderSyncStarted",
        "slotFolderSyncFinished",
        "SyncResult",
        "slotRemoveFoldersForAccount",
        "AccountState*",
        "accountState",
        "slotServerVersionChanged",
        "Account*",
        "account",
        "saveFolder"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'folderSyncStateChange'
        QtMocHelpers::SignalData<void(Folder *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'folderListChanged'
        QtMocHelpers::SignalData<void(const QUuid &, const QList<Folder*>)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'folderAdded'
        QtMocHelpers::SignalData<void(const QUuid &, Folder *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 3, 10 },
        }}),
        // Signal 'folderRemoved'
        QtMocHelpers::SignalData<void(const QUuid &, Folder *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 3, 10 },
        }}),
        // Signal 'unsyncedSpaceCountChanged'
        QtMocHelpers::SignalData<void(QUuid, int, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'slotIsConnectedChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncOnceFileUnlocks'
        QtMocHelpers::SlotData<void(const QString &, FileSystem::LockMode)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotReloadSyncOptions'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeFolderFromGui'
        QtMocHelpers::SlotData<void(Folder *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'forceFolderSync'
        QtMocHelpers::SlotData<void(Folder *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'slotFolderSyncPauseChanged'
        QtMocHelpers::SlotData<void(Folder *, bool)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'slotFolderCanSyncChanged'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderSyncStarted'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderSyncFinished'
        QtMocHelpers::SlotData<void(const SyncResult &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 2 },
        }}),
        // Slot 'slotRemoveFoldersForAccount'
        QtMocHelpers::SlotData<void(AccountState *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'slotServerVersionChanged'
        QtMocHelpers::SlotData<void(Account *)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'saveFolder'
        QtMocHelpers::SlotData<void(Folder *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderMan, qt_meta_tag_ZN3OCC9FolderManE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderMan::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9FolderManE_t>.metaTypes,
    nullptr
} };

void OCC::FolderMan::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderMan *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->folderSyncStateChange((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 1: _t->folderListChanged((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Folder*>>>(_a[2]))); break;
        case 2: _t->folderAdded((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[2]))); break;
        case 3: _t->folderRemoved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[2]))); break;
        case 4: _t->unsyncedSpaceCountChanged((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->slotIsConnectedChanged(); break;
        case 6: _t->slotSyncOnceFileUnlocks((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<FileSystem::LockMode>>(_a[2]))); break;
        case 7: _t->slotReloadSyncOptions(); break;
        case 8: _t->removeFolderFromGui((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 9: _t->forceFolderSync((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 10: _t->slotFolderSyncPauseChanged((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->slotFolderCanSyncChanged(); break;
        case 12: _t->slotFolderSyncStarted(); break;
        case 13: _t->slotFolderSyncFinished((*reinterpret_cast<std::add_pointer_t<SyncResult>>(_a[1]))); break;
        case 14: _t->slotRemoveFoldersForAccount((*reinterpret_cast<std::add_pointer_t<AccountState*>>(_a[1]))); break;
        case 15: _t->slotServerVersionChanged((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 16: _t->saveFolder((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Folder*> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AccountState* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(Folder * )>(_a, &FolderMan::folderSyncStateChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(const QUuid & , const QList<Folder*> )>(_a, &FolderMan::folderListChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(const QUuid & , Folder * )>(_a, &FolderMan::folderAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(const QUuid & , Folder * )>(_a, &FolderMan::folderRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(QUuid , int , int )>(_a, &FolderMan::unsyncedSpaceCountChanged, 4))
            return;
    }
}

const QMetaObject *OCC::FolderMan::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderMan::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::FolderMan::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void OCC::FolderMan::folderSyncStateChange(Folder * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::FolderMan::folderListChanged(const QUuid & _t1, const QList<Folder*> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::FolderMan::folderAdded(const QUuid & _t1, Folder * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::FolderMan::folderRemoved(const QUuid & _t1, Folder * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::FolderMan::unsyncedSpaceCountChanged(QUuid _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
