/****************************************************************************
** Meta object code from reading C++ file 'theme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/theme.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'theme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12QmlUrlButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::QmlUrlButton::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12QmlUrlButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::QmlUrlButton",
        "QML.Element",
        "urlbuttondata",
        "icon",
        "name",
        "url",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'icon'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QmlUrlButton, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::QmlUrlButton::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12QmlUrlButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12QmlUrlButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12QmlUrlButtonE_t>.metaTypes,
    nullptr
} };

void OCC::QmlUrlButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QmlUrlButton *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->icon; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name; break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->url; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC14QmlButtonColorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::QmlButtonColor::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14QmlButtonColorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::QmlButtonColor",
        "QML.Element",
        "buttonColor",
        "color",
        "QColor",
        "textColor",
        "textColorDisabled",
        "valid"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'textColor'
        QtMocHelpers::PropertyData<QColor>(5, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'textColorDisabled'
        QtMocHelpers::PropertyData<QColor>(6, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QmlButtonColor, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::QmlButtonColor::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14QmlButtonColorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14QmlButtonColorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14QmlButtonColorE_t>.metaTypes,
    nullptr
} };

void OCC::QmlButtonColor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QmlButtonColor *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color; break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->textColor; break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->textColorDisabled; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->valid(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC5ThemeE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Theme::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC5ThemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Theme",
        "QML.Singleton",
        "true",
        "QML.Element",
        "auto",
        "systrayUseMonoIconsChanged",
        "",
        "themeChanged",
        "multiAccount",
        "singleSyncFolder",
        "urlButtons",
        "QList<QmlUrlButton>",
        "syncNewlyDiscoveredSpaces",
        "avatarColor",
        "QColor",
        "avatarColorChecked",
        "brandedBackgoundColor",
        "brandedForegroundColor",
        "primaryButtonColor",
        "QmlButtonColor",
        "secondaryButtonColor",
        "spacesAreCalledFolders",
        "VersionFormat",
        "Plain",
        "Url",
        "RichText",
        "OneLiner",
        "UserIDType",
        "UserIDUserName",
        "UserIDEmail",
        "UserIDCustom"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'systrayUseMonoIconsChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'multiAccount'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'singleSyncFolder'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'urlButtons'
        QtMocHelpers::PropertyData<QList<QmlUrlButton>>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'syncNewlyDiscoveredSpaces'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'avatarColor'
        QtMocHelpers::PropertyData<QColor>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'avatarColorChecked'
        QtMocHelpers::PropertyData<QColor>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'brandedBackgoundColor'
        QtMocHelpers::PropertyData<QColor>(16, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'brandedForegroundColor'
        QtMocHelpers::PropertyData<QColor>(17, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'primaryButtonColor'
        QtMocHelpers::PropertyData<QmlButtonColor>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'secondaryButtonColor'
        QtMocHelpers::PropertyData<QmlButtonColor>(20, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'spacesAreCalledFolders'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VersionFormat'
        QtMocHelpers::EnumData<enum VersionFormat>(22, 22, QMC::EnumIsScoped).add({
            {   23, VersionFormat::Plain },
            {   24, VersionFormat::Url },
            {   25, VersionFormat::RichText },
            {   26, VersionFormat::OneLiner },
        }),
        // enum 'UserIDType'
        QtMocHelpers::EnumData<enum UserIDType>(27, 27, QMC::EnumFlags{}).add({
            {   28, UserIDType::UserIDUserName },
            {   29, UserIDType::UserIDEmail },
            {   30, UserIDType::UserIDCustom },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Theme, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::Theme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC5ThemeE_t>.metaTypes,
    nullptr
} };

void OCC::Theme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Theme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->systrayUseMonoIconsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->themeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Theme::*)(bool )>(_a, &Theme::systrayUseMonoIconsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::themeChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->multiAccount(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->singleSyncFolder(); break;
        case 2: *reinterpret_cast<QList<QmlUrlButton>*>(_v) = _t->qmlUrlButtons(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->syncNewlyDiscoveredSpaces(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->avatarColor(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->avatarColorChecked(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->wizardHeaderBackgroundColor(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->wizardHeaderTitleColor(); break;
        case 8: *reinterpret_cast<QmlButtonColor*>(_v) = _t->primaryButtonColor(); break;
        case 9: *reinterpret_cast<QmlButtonColor*>(_v) = _t->secondaryButtonColor(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->spacesAreCalledFolders(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Theme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Theme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Theme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OCC::Theme::systrayUseMonoIconsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::Theme::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
