/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<account.h>)
#  include <account.h>
#endif
#if __has_include(<space.h>)
#  include <space.h>
#endif
#if __has_include(<theme.h>)
#  include <theme.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
namespace OCC {
Q_QMLTYPE_EXPORT void qml_register_types_org_ownCloud_libsync()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<OCC::Account>("org.ownCloud.libsync", 1);
    qmlRegisterEnum<OCC::Account::ServerSupportLevel>("OCC::Account::ServerSupportLevel");
    qmlRegisterTypesAndRevisions<OCC::GraphApi::Space>("org.ownCloud.libsync", 1);
    qmlRegisterTypesAndRevisions<OCC::GraphApi::SpaceImage>("org.ownCloud.libsync", 1);
    qmlRegisterTypesAndRevisions<OCC::QmlButtonColor>("org.ownCloud.libsync", 1);
    qmlRegisterTypesAndRevisions<OCC::QmlUrlButton>("org.ownCloud.libsync", 1);
    qmlRegisterTypesAndRevisions<OCC::Theme>("org.ownCloud.libsync", 1);
    qmlRegisterEnum<OCC::Theme::VersionFormat>("OCC::Theme::VersionFormat");
    qmlRegisterEnum<OCC::Theme::UserIDType>("OCC::Theme::UserIDType");
    QT_WARNING_POP
    qmlRegisterModule("org.ownCloud.libsync", 1, 0);
}

static const QQmlModuleRegistration orgownCloudlibsyncRegistration("org.ownCloud.libsync", qml_register_types_org_ownCloud_libsync);
} // namespace OCC
