/********************************************************************************
** Form generated from reading UI file 'accountmodalwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTMODALWIDGET_H
#define UI_ACCOUNTMODALWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace OCC {

class Ui_AccountModalWidget
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *OCC__AccountModalWidget)
    {
        if (OCC__AccountModalWidget->objectName().isEmpty())
            OCC__AccountModalWidget->setObjectName("OCC__AccountModalWidget");
        OCC__AccountModalWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(OCC__AccountModalWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(25, -1, 25, -1);
        groupBox = new QGroupBox(OCC__AccountModalWidget);
        groupBox->setObjectName("groupBox");
        groupBox->setTitle(QString::fromUtf8("placeholder"));
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");

        verticalLayout->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(OCC__AccountModalWidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::NoButton);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(OCC__AccountModalWidget);

        QMetaObject::connectSlotsByName(OCC__AccountModalWidget);
    } // setupUi

    void retranslateUi(QWidget *OCC__AccountModalWidget)
    {
        OCC__AccountModalWidget->setWindowTitle(QCoreApplication::translate("OCC::AccountModalWidget", "Form", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class AccountModalWidget: public Ui_AccountModalWidget {};
} // namespace Ui
} // namespace OCC

#endif // UI_ACCOUNTMODALWIDGET_H
