/*
 * Copyright (C) by Hannah von Reth <hannah.vonreth@owncloud.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 */

#pragma once

#include "libsync/graphapi/space.h"

#include <QObject>

#include <unordered_map>

namespace OCC {

class FolderMan;
class Folder;

class ETagWatcher : public QObject
{
    Q_OBJECT
public:
    ETagWatcher(QObject *parent);

public Q_SLOTS:
    void slotSpaceChanged(GraphApi::Space *space);

    void slotFolderListChanged(const QUuid &accountId, const QList<Folder *> folders);

    void onFolderAdded(const QUuid &accountId, Folder *folder);

    void onFolderRemoved(const QUuid &accountId, Folder *folder);


Q_SIGNALS:
    void requestEnqueueFolder(Folder *folder);

private:
    void updateEtag(const QUuid &accountId, const QString &spaceId, const QString &etag);

    struct ETagInfo
    {
        QString etag;
        Folder *folder;
    };

    // we have to separate the data into accounts because it's NOT the case that all spaceid's are unique
    // specifically, the Shares space always has the same space id -> see Space.cpp sharesIdC
    // so we have a hash keyed on the uuid of the account, which in turn holds the hash for the space ids->etag info
    QHash<QUuid, QHash<QString, ETagInfo>> _lastEtagJobForSpace;
};

}
