/********************************************************************************
** Form generated from reading UI file 'tlserrordialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TLSERRORDIALOG_H
#define UI_TLSERRORDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

namespace OCC {

class Ui_TlsErrorDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *hostnameLabel;
    QTextBrowser *textBrowser;
    QLabel *label;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OCC__TlsErrorDialog)
    {
        if (OCC__TlsErrorDialog->objectName().isEmpty())
            OCC__TlsErrorDialog->setObjectName("OCC__TlsErrorDialog");
        OCC__TlsErrorDialog->resize(534, 493);
        verticalLayout = new QVBoxLayout(OCC__TlsErrorDialog);
        verticalLayout->setObjectName("verticalLayout");
        hostnameLabel = new QLabel(OCC__TlsErrorDialog);
        hostnameLabel->setObjectName("hostnameLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(hostnameLabel->sizePolicy().hasHeightForWidth());
        hostnameLabel->setSizePolicy(sizePolicy);
        hostnameLabel->setText(QString::fromUtf8("Cannot connect securely to host hostname (placeholder)"));

        verticalLayout->addWidget(hostnameLabel);

        textBrowser = new QTextBrowser(OCC__TlsErrorDialog);
        textBrowser->setObjectName("textBrowser");

        verticalLayout->addWidget(textBrowser);

        label = new QLabel(OCC__TlsErrorDialog);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(label);

        buttonBox = new QDialogButtonBox(OCC__TlsErrorDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::No|QDialogButtonBox::Yes);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(OCC__TlsErrorDialog);

        QMetaObject::connectSlotsByName(OCC__TlsErrorDialog);
    } // setupUi

    void retranslateUi(QDialog *OCC__TlsErrorDialog)
    {
        OCC__TlsErrorDialog->setWindowTitle(QCoreApplication::translate("OCC::TlsErrorDialog", "TLS Certificate Error", nullptr));
        label->setText(QCoreApplication::translate("OCC::TlsErrorDialog", "Do you want to trust this certificate anyway?", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class TlsErrorDialog: public Ui_TlsErrorDialog {};
} // namespace Ui
} // namespace OCC

#endif // UI_TLSERRORDIALOG_H
