/****************************************************************************
** Meta object code from reading C++ file 'account.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/account.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC7AccountE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Account::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC7AccountE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Account",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only created in the C++ code",
        "invalidCredentials",
        "",
        "credentialsFetched",
        "wantsAccountSaved",
        "Account*",
        "acc",
        "serverVersionChanged",
        "avatarChanged",
        "displayNameChanged",
        "unknownConnectionState",
        "appProviderErrorOccured",
        "error",
        "clearAMCache",
        "uid",
        "QUuid",
        "davUser",
        "davDisplayName",
        "displayNameWithHost",
        "initials",
        "hostName",
        "hasAvatar",
        "avatarGradient",
        "QGradient::Preset",
        "url",
        "QUrl",
        "ServerSupportLevel",
        "Supported",
        "Unknown",
        "Unsupported"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invalidCredentials'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'credentialsFetched'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wantsAccountSaved'
        QtMocHelpers::SignalData<void(Account *)>(10, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'serverVersionChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'avatarChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayNameChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unknownConnectionState'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'appProviderErrorOccured'
        QtMocHelpers::SignalData<void(const QString &)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'clearAMCache'
        QtMocHelpers::SlotData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'uid'
        QtMocHelpers::PropertyData<QUuid>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'davUser'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'davDisplayName'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'displayNameWithHost'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'initials'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'hostName'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'hasAvatar'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'avatarGradient'
        QtMocHelpers::PropertyData<QGradient::Preset>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ServerSupportLevel'
        QtMocHelpers::EnumData<enum ServerSupportLevel>(32, 32, QMC::EnumIsScoped).add({
            {   33, ServerSupportLevel::Supported },
            {   34, ServerSupportLevel::Unknown },
            {   35, ServerSupportLevel::Unsupported },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Account, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC7AccountE[] = {
    QMetaObject::SuperData::link<QGradient::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::Account::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7AccountE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7AccountE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC7AccountE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC7AccountE_t>.metaTypes,
    nullptr
} };

void OCC::Account::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Account *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invalidCredentials(QPrivateSignal()); break;
        case 1: _t->credentialsFetched(); break;
        case 2: _t->wantsAccountSaved((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 3: _t->serverVersionChanged(); break;
        case 4: _t->avatarChanged(); break;
        case 5: _t->displayNameChanged(); break;
        case 6: _t->unknownConnectionState(); break;
        case 7: _t->appProviderErrorOccured((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->clearAMCache(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Account::*)(QPrivateSignal)>(_a, &Account::invalidCredentials, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::credentialsFetched, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)(Account * )>(_a, &Account::wantsAccountSaved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::serverVersionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::avatarChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::displayNameChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::unknownConnectionState, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)(const QString & )>(_a, &Account::appProviderErrorOccured, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUuid*>(_v) = _t->uuid(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->davUser(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->davDisplayName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->displayNameWithHost(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->initials(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->hostName(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hasAvatar(); break;
        case 7: *reinterpret_cast<QGradient::Preset*>(_v) = _t->avatarGradient(); break;
        case 8: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Account::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Account::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7AccountE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Account::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OCC::Account::invalidCredentials(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::Account::credentialsFetched()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Account::wantsAccountSaved(Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::Account::serverVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Account::avatarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Account::displayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Account::unknownConnectionState()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::Account::appProviderErrorOccured(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
