/****************************************************************************
** Meta object code from reading C++ file 'newaccountmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/newaccountwizard/newaccountmodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'newaccountmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC15NewAccountModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::NewAccountModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC15NewAccountModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::NewAccountModel",
        "serverUrlChanged",
        "",
        "QUrl",
        "newUrl",
        "webfingerAuthenticationUrlChanged",
        "newWebfingerAuthenticationUrl",
        "webfingerUserInfoUrlChanged",
        "newWebfingerUserUrl",
        "trustedCertificatesChanged",
        "QSet<QSslCertificate>",
        "trustedCertificates",
        "displayNameChanged",
        "displayName",
        "davUserChanged",
        "davUser",
        "authTokenChanged",
        "authToken",
        "refreshTokenChanged",
        "refreshToken",
        "capabilitiesChanged",
        "OCC::Capabilities",
        "capabilities",
        "defaultSyncRootChanged",
        "defaultSyncRoot",
        "syncTypeChanged",
        "NewAccount::SyncType",
        "syncType",
        "serverUrl",
        "webfingerAuthenticationUrl",
        "webfingerUserInfoUrl",
        "Capabilities"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'webfingerAuthenticationUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'webfingerUserInfoUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Signal 'trustedCertificatesChanged'
        QtMocHelpers::SignalData<void(QSet<QSslCertificate>)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'displayNameChanged'
        QtMocHelpers::SignalData<void(QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'davUserChanged'
        QtMocHelpers::SignalData<void(QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'authTokenChanged'
        QtMocHelpers::SignalData<void(QString)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'refreshTokenChanged'
        QtMocHelpers::SignalData<void(QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'capabilitiesChanged'
        QtMocHelpers::SignalData<void(OCC::Capabilities)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'defaultSyncRootChanged'
        QtMocHelpers::SignalData<void(QString)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'syncTypeChanged'
        QtMocHelpers::SignalData<void(NewAccount::SyncType)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'serverUrl'
        QtMocHelpers::PropertyData<QUrl>(28, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'webfingerAuthenticationUrl'
        QtMocHelpers::PropertyData<QUrl>(29, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'webfingerUserInfoUrl'
        QtMocHelpers::PropertyData<QUrl>(30, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'trustedCertificates'
        QtMocHelpers::PropertyData<QSet<QSslCertificate>>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'davUser'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'authToken'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'refreshToken'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'defaultSyncRoot'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'syncType'
        QtMocHelpers::PropertyData<NewAccount::SyncType>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'capabilities'
        QtMocHelpers::PropertyData<Capabilities>(22, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NewAccountModel, qt_meta_tag_ZN3OCC15NewAccountModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC15NewAccountModelE[] = {
    QMetaObject::SuperData::link<OCC::NewAccount::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::NewAccountModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15NewAccountModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15NewAccountModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC15NewAccountModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC15NewAccountModelE_t>.metaTypes,
    nullptr
} };

void OCC::NewAccountModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NewAccountModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->webfingerAuthenticationUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->webfingerUserInfoUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->trustedCertificatesChanged((*reinterpret_cast<std::add_pointer_t<QSet<QSslCertificate>>>(_a[1]))); break;
        case 4: _t->displayNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->davUserChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->authTokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->refreshTokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->capabilitiesChanged((*reinterpret_cast<std::add_pointer_t<OCC::Capabilities>>(_a[1]))); break;
        case 9: _t->defaultSyncRootChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->syncTypeChanged((*reinterpret_cast<std::add_pointer_t<NewAccount::SyncType>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QSslCertificate> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(const QUrl & )>(_a, &NewAccountModel::serverUrlChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(const QUrl & )>(_a, &NewAccountModel::webfingerAuthenticationUrlChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(const QUrl & )>(_a, &NewAccountModel::webfingerUserInfoUrlChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QSet<QSslCertificate> )>(_a, &NewAccountModel::trustedCertificatesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QString )>(_a, &NewAccountModel::displayNameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QString )>(_a, &NewAccountModel::davUserChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QString )>(_a, &NewAccountModel::authTokenChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QString )>(_a, &NewAccountModel::refreshTokenChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(OCC::Capabilities )>(_a, &NewAccountModel::capabilitiesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(QString )>(_a, &NewAccountModel::defaultSyncRootChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (NewAccountModel::*)(NewAccount::SyncType )>(_a, &NewAccountModel::syncTypeChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSet<QSslCertificate> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->serverUrl(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->webfingerAuthenticationUrl(); break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->webfingerUserInfoUrl(); break;
        case 3: *reinterpret_cast<QSet<QSslCertificate>*>(_v) = _t->trustedCertificates(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->displayName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->davUser(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->authToken(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->refreshToken(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->defaultSyncRoot(); break;
        case 9: *reinterpret_cast<NewAccount::SyncType*>(_v) = _t->syncType(); break;
        case 10: *reinterpret_cast<Capabilities*>(_v) = _t->capabilities(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServerUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 1: _t->setWebfingerAuthenticationUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 2: _t->setWebfingerUserInfoUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 3: _t->setTrustedCertificates(*reinterpret_cast<QSet<QSslCertificate>*>(_v)); break;
        case 4: _t->setDisplayName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setDavUser(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setAuthToken(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setRefreshToken(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setDefaultSyncRoot(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setSyncType(*reinterpret_cast<NewAccount::SyncType*>(_v)); break;
        case 10: _t->setCapabilities(*reinterpret_cast<Capabilities*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::NewAccountModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::NewAccountModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15NewAccountModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::NewAccountModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OCC::NewAccountModel::serverUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::NewAccountModel::webfingerAuthenticationUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::NewAccountModel::webfingerUserInfoUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::NewAccountModel::trustedCertificatesChanged(QSet<QSslCertificate> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::NewAccountModel::displayNameChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::NewAccountModel::davUserChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void OCC::NewAccountModel::authTokenChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void OCC::NewAccountModel::refreshTokenChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void OCC::NewAccountModel::capabilitiesChanged(OCC::Capabilities _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void OCC::NewAccountModel::defaultSyncRootChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void OCC::NewAccountModel::syncTypeChanged(NewAccount::SyncType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
