/****************************************************************************
** Meta object code from reading C++ file 'owncloudgui.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/owncloudgui.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'owncloudgui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC11ownCloudGuiE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ownCloudGui::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ownCloudGui",
        "requestSetUpSyncFoldersForAccount",
        "",
        "AccountState*",
        "account",
        "useVfs",
        "requestLoadSpacesOnly",
        "setupTrayContextMenu",
        "slotComputeOverallSyncStatus",
        "slotShowTrayMessage",
        "title",
        "msg",
        "QIcon",
        "icon",
        "slotShowOptionalTrayMessage",
        "slotShowSettings",
        "slotShutdown",
        "slotSyncStateChange",
        "Folder*",
        "slotTrayClicked",
        "QSystemTrayIcon::ActivationReason",
        "reason",
        "slotToggleLogBrowser",
        "slotOpenSettingsDialog",
        "slotHelp",
        "slotAbout",
        "slotTrayMessageIfServerUnsupported",
        "Account*",
        "slotShowShareInBrowser",
        "sharePath",
        "localPath",
        "handleAccountSetupError",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSetUpSyncFoldersForAccount'
        QtMocHelpers::SignalData<void(AccountState *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'requestLoadSpacesOnly'
        QtMocHelpers::SignalData<void(AccountState *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setupTrayContextMenu'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotComputeOverallSyncStatus'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowTrayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QIcon &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotShowTrayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slotShowOptionalTrayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QIcon &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotShowOptionalTrayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slotShowSettings'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShutdown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncStateChange'
        QtMocHelpers::SlotData<void(Folder *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Slot 'slotTrayClicked'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotToggleLogBrowser'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenSettingsDialog'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHelp'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAbout'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTrayMessageIfServerUnsupported'
        QtMocHelpers::SlotData<void(Account *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 4 },
        }}),
        // Slot 'slotShowShareInBrowser'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { QMetaType::QString, 30 },
        }}),
        // Slot 'handleAccountSetupError'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ownCloudGui, qt_meta_tag_ZN3OCC11ownCloudGuiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ownCloudGui::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.metaTypes,
    nullptr
} };

void OCC::ownCloudGui::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ownCloudGui *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSetUpSyncFoldersForAccount((*reinterpret_cast<std::add_pointer_t<AccountState*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->requestLoadSpacesOnly((*reinterpret_cast<std::add_pointer_t<AccountState*>>(_a[1]))); break;
        case 2: _t->setupTrayContextMenu(); break;
        case 3: _t->slotComputeOverallSyncStatus(); break;
        case 4: _t->slotShowTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[3]))); break;
        case 5: _t->slotShowTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->slotShowOptionalTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[3]))); break;
        case 7: _t->slotShowOptionalTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->slotShowSettings(); break;
        case 9: _t->slotShutdown(); break;
        case 10: _t->slotSyncStateChange((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 11: _t->slotTrayClicked((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        case 12: _t->slotToggleLogBrowser(); break;
        case 13: _t->slotOpenSettingsDialog(); break;
        case 14: _t->slotHelp(); break;
        case 15: _t->slotAbout(); break;
        case 16: _t->slotTrayMessageIfServerUnsupported((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 17: _t->slotShowShareInBrowser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->handleAccountSetupError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ownCloudGui::*)(AccountState * , bool )>(_a, &ownCloudGui::requestSetUpSyncFoldersForAccount, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ownCloudGui::*)(AccountState * )>(_a, &ownCloudGui::requestLoadSpacesOnly, 1))
            return;
    }
}

const QMetaObject *OCC::ownCloudGui::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ownCloudGui::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ownCloudGui::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void OCC::ownCloudGui::requestSetUpSyncFoldersForAccount(AccountState * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::ownCloudGui::requestLoadSpacesOnly(AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
