/********************************************************************************
** Form generated from reading UI file 'localactivitywidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCALACTIVITYWIDGET_H
#define UI_LOCALACTIVITYWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace OCC {

class Ui_LocalActivityWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *_filterButton;
    QTableView *_tableView;

    void setupUi(QWidget *OCC__LocalActivityWidget)
    {
        if (OCC__LocalActivityWidget->objectName().isEmpty())
            OCC__LocalActivityWidget->setObjectName("OCC__LocalActivityWidget");
        OCC__LocalActivityWidget->resize(612, 515);
        verticalLayout = new QVBoxLayout(OCC__LocalActivityWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        _filterButton = new QPushButton(OCC__LocalActivityWidget);
        _filterButton->setObjectName("_filterButton");

        horizontalLayout->addWidget(_filterButton);


        verticalLayout->addLayout(horizontalLayout);

        _tableView = new QTableView(OCC__LocalActivityWidget);
        _tableView->setObjectName("_tableView");
        _tableView->setContextMenuPolicy(Qt::CustomContextMenu);
        _tableView->setTabKeyNavigation(false);
        _tableView->setAlternatingRowColors(true);
        _tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
        _tableView->setShowGrid(false);
        _tableView->setSortingEnabled(true);
        _tableView->setWordWrap(false);
        _tableView->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(_tableView);

        QWidget::setTabOrder(_filterButton, _tableView);

        retranslateUi(OCC__LocalActivityWidget);

        QMetaObject::connectSlotsByName(OCC__LocalActivityWidget);
    } // setupUi

    void retranslateUi(QWidget *OCC__LocalActivityWidget)
    {
        _filterButton->setText(QCoreApplication::translate("OCC::LocalActivityWidget", "Filter", nullptr));
#if QT_CONFIG(accessibility)
        _tableView->setAccessibleName(QCoreApplication::translate("OCC::LocalActivityWidget", "Local activity table", nullptr));
#endif // QT_CONFIG(accessibility)
        (void)OCC__LocalActivityWidget;
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class LocalActivityWidget: public Ui_LocalActivityWidget {};
} // namespace Ui
} // namespace OCC

#endif // UI_LOCALACTIVITYWIDGET_H
