/****************************************************************************
** Meta object code from reading C++ file 'syncerrorwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/gui/syncerrorwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncerrorwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncErrorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncErrorWidget",
        "issueCountUpdated",
        "",
        "slotProgressInfo",
        "Folder*",
        "folder",
        "ProgressInfo",
        "progress",
        "slotItemCompleted",
        "SyncFileItemPtr",
        "item",
        "filterChanged",
        "showFilterMenu",
        "QMenu*",
        "QWidget*",
        "parent",
        "slotItemContextMenu",
        "QPoint",
        "pos",
        "onFolderListChanged",
        "QUuid",
        "accountId",
        "QList<Folder*>",
        "folders",
        "onFolderRemoved",
        "f"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'issueCountUpdated'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotProgressInfo'
        QtMocHelpers::SlotData<void(Folder *, const ProgressInfo &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(Folder *, const SyncFileItemPtr &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'filterChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showFilterMenu'
        QtMocHelpers::SlotData<QMenu *(QWidget *)>(12, 2, QMC::AccessPrivate, 0x80000000 | 13, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotItemContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'onFolderListChanged'
        QtMocHelpers::SlotData<void(const QUuid &, const QList<Folder*>)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'onFolderRemoved'
        QtMocHelpers::SlotData<void(const QUuid &, Folder *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 4, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SyncErrorWidget, qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncErrorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>.metaTypes,
    nullptr
} };

void OCC::SyncErrorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncErrorWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->issueCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->slotProgressInfo((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ProgressInfo>>(_a[2]))); break;
        case 2: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[2]))); break;
        case 3: _t->filterChanged(); break;
        case 4: { QMenu* _r = _t->showFilterMenu((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QMenu**>(_a[0]) = std::move(_r); }  break;
        case 5: _t->slotItemContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 6: _t->onFolderListChanged((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Folder*>>>(_a[2]))); break;
        case 7: _t->onFolderRemoved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncErrorWidget::*)(int )>(_a, &SyncErrorWidget::issueCountUpdated, 0))
            return;
    }
}

const QMetaObject *OCC::SyncErrorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SyncErrorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15SyncErrorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int OCC::SyncErrorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void OCC::SyncErrorWidget::issueCountUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
