/****************************************************************************
** Meta object code from reading C++ file 'discoveryphase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ownCloud-7.1.0.19041/src/libsync/discoveryphase.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'discoveryphase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoverySingleLocalDirectoryJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoverySingleLocalDirectoryJob",
        "finished",
        "",
        "QList<LocalInfo>",
        "result",
        "finishedFatalError",
        "errorString",
        "finishedNonFatalError",
        "itemDiscovered",
        "SyncFileItemPtr",
        "item",
        "childIgnored",
        "b"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void(QVector<LocalInfo>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'finishedFatalError'
        QtMocHelpers::SignalData<void(QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'finishedNonFatalError'
        QtMocHelpers::SignalData<void(QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'itemDiscovered'
        QtMocHelpers::SignalData<void(SyncFileItemPtr)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'childIgnored'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoverySingleLocalDirectoryJob, qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoverySingleLocalDirectoryJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoverySingleLocalDirectoryJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoverySingleLocalDirectoryJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast<std::add_pointer_t<QList<LocalInfo>>>(_a[1]))); break;
        case 1: _t->finishedFatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->finishedNonFatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->itemDiscovered((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 4: _t->childIgnored((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QVector<LocalInfo> )>(_a, &DiscoverySingleLocalDirectoryJob::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QString )>(_a, &DiscoverySingleLocalDirectoryJob::finishedFatalError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QString )>(_a, &DiscoverySingleLocalDirectoryJob::finishedNonFatalError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(SyncFileItemPtr )>(_a, &DiscoverySingleLocalDirectoryJob::itemDiscovered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(bool )>(_a, &DiscoverySingleLocalDirectoryJob::childIgnored, 4))
            return;
    }
}

const QMetaObject *OCC::DiscoverySingleLocalDirectoryJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoverySingleLocalDirectoryJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QRunnable"))
        return static_cast< QRunnable*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoverySingleLocalDirectoryJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoverySingleLocalDirectoryJob::finished(QVector<LocalInfo> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::DiscoverySingleLocalDirectoryJob::finishedFatalError(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::DiscoverySingleLocalDirectoryJob::finishedNonFatalError(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::DiscoverySingleLocalDirectoryJob::itemDiscovered(SyncFileItemPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::DiscoverySingleLocalDirectoryJob::childIgnored(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoverySingleDirectoryJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoverySingleDirectoryJob",
        "firstDirectoryPermissions",
        "",
        "RemotePermissions",
        "etag",
        "time",
        "finished",
        "HttpResult<QList<RemoteInfo>>",
        "result",
        "directoryListingIteratedSlot",
        "QMap<QString,QString>",
        "lsJobFinishedWithoutErrorSlot"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'firstDirectoryPermissions'
        QtMocHelpers::SignalData<void(RemotePermissions)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'etag'
        QtMocHelpers::SignalData<void(const QString &, const QDateTime &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QDateTime, 5 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(const HttpResult<QVector<RemoteInfo>> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'directoryListingIteratedSlot'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 10, 2 },
        }}),
        // Slot 'lsJobFinishedWithoutErrorSlot'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoverySingleDirectoryJob, qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoverySingleDirectoryJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoverySingleDirectoryJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoverySingleDirectoryJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->firstDirectoryPermissions((*reinterpret_cast<std::add_pointer_t<RemotePermissions>>(_a[1]))); break;
        case 1: _t->etag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<HttpResult<QList<RemoteInfo>>>>(_a[1]))); break;
        case 3: _t->directoryListingIteratedSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 4: _t->lsJobFinishedWithoutErrorSlot(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(RemotePermissions )>(_a, &DiscoverySingleDirectoryJob::firstDirectoryPermissions, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(const QString & , const QDateTime & )>(_a, &DiscoverySingleDirectoryJob::etag, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(const HttpResult<QVector<RemoteInfo>> & )>(_a, &DiscoverySingleDirectoryJob::finished, 2))
            return;
    }
}

const QMetaObject *OCC::DiscoverySingleDirectoryJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoverySingleDirectoryJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoverySingleDirectoryJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoverySingleDirectoryJob::firstDirectoryPermissions(RemotePermissions _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::DiscoverySingleDirectoryJob::etag(const QString & _t1, const QDateTime & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::DiscoverySingleDirectoryJob::finished(const HttpResult<QVector<RemoteInfo>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoveryPhase::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoveryPhase",
        "fatalError",
        "",
        "errorString",
        "itemDiscovered",
        "SyncFileItemPtr",
        "item",
        "finished",
        "silentlyExcluded",
        "folderPath",
        "excluded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fatalError'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'itemDiscovered'
        QtMocHelpers::SignalData<void(const SyncFileItemPtr &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'silentlyExcluded'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'excluded'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoveryPhase, qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoveryPhase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoveryPhase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoveryPhase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->itemDiscovered((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 2: _t->finished(); break;
        case 3: _t->silentlyExcluded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->excluded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & )>(_a, &DiscoveryPhase::fatalError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const SyncFileItemPtr & )>(_a, &DiscoveryPhase::itemDiscovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)()>(_a, &DiscoveryPhase::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & )>(_a, &DiscoveryPhase::silentlyExcluded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & )>(_a, &DiscoveryPhase::excluded, 4))
            return;
    }
}

const QMetaObject *OCC::DiscoveryPhase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoveryPhase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoveryPhase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoveryPhase::fatalError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::DiscoveryPhase::itemDiscovered(const SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::DiscoveryPhase::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::DiscoveryPhase::silentlyExcluded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::DiscoveryPhase::excluded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
