/********************************************************************************
** Form generated from reading UI file 'generalsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENERALSETTINGS_H
#define UI_GENERALSETTINGS_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "networksettings.h"

namespace OCC {

class Ui_GeneralSettings
{
public:
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *generalGroupBox;
    QGridLayout *gridLayout_2;
    QCheckBox *monoIconsCheckBox;
    QCheckBox *desktopNotificationsCheckBox;
    QCheckBox *autostartCheckBox;
    QHBoxLayout *horizontalLayout_6;
    QLabel *languageLabel;
    QComboBox *languageDropdown;
    QGroupBox *advancedGroupBox;
    QHBoxLayout *horizontalLayout_5;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *syncHiddenFilesCheckBox;
    QCheckBox *crashreporterCheckBox;
    QCheckBox *moveToTrashCheckBox;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *ignoredFilesButton;
    QPushButton *logSettingsButton;
    QSpacerItem *spacer_3;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    NetworkSettings *widget;
    QSpacerItem *verticalSpacer;
    QWidget *versionInfoWidget;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *spacer;
    QLabel *copyrightLabel;
    QPushButton *about_pushButton;

    void setupUi(QWidget *OCC__GeneralSettings)
    {
        if (OCC__GeneralSettings->objectName().isEmpty())
            OCC__GeneralSettings->setObjectName("OCC__GeneralSettings");
        OCC__GeneralSettings->resize(791, 686);
        verticalLayout_3 = new QVBoxLayout(OCC__GeneralSettings);
        verticalLayout_3->setObjectName("verticalLayout_3");
        scrollArea = new QScrollArea(OCC__GeneralSettings);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 765, 618));
        verticalLayout_5 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        generalGroupBox = new QGroupBox(scrollAreaWidgetContents_2);
        generalGroupBox->setObjectName("generalGroupBox");
        gridLayout_2 = new QGridLayout(generalGroupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        monoIconsCheckBox = new QCheckBox(generalGroupBox);
        monoIconsCheckBox->setObjectName("monoIconsCheckBox");

        gridLayout_2->addWidget(monoIconsCheckBox, 1, 0, 1, 1);

        desktopNotificationsCheckBox = new QCheckBox(generalGroupBox);
        desktopNotificationsCheckBox->setObjectName("desktopNotificationsCheckBox");

        gridLayout_2->addWidget(desktopNotificationsCheckBox, 0, 1, 1, 1);

        autostartCheckBox = new QCheckBox(generalGroupBox);
        autostartCheckBox->setObjectName("autostartCheckBox");

        gridLayout_2->addWidget(autostartCheckBox, 0, 0, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        languageLabel = new QLabel(generalGroupBox);
        languageLabel->setObjectName("languageLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(languageLabel->sizePolicy().hasHeightForWidth());
        languageLabel->setSizePolicy(sizePolicy);
        languageLabel->setLocale(QLocale(QLocale::English, QLocale::UnitedStates));

        horizontalLayout_6->addWidget(languageLabel);

        languageDropdown = new QComboBox(generalGroupBox);
        languageDropdown->setObjectName("languageDropdown");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(languageDropdown->sizePolicy().hasHeightForWidth());
        languageDropdown->setSizePolicy(sizePolicy1);

        horizontalLayout_6->addWidget(languageDropdown);


        gridLayout_2->addLayout(horizontalLayout_6, 2, 0, 1, 1);


        verticalLayout_5->addWidget(generalGroupBox);

        advancedGroupBox = new QGroupBox(scrollAreaWidgetContents_2);
        advancedGroupBox->setObjectName("advancedGroupBox");
        horizontalLayout_5 = new QHBoxLayout(advancedGroupBox);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        syncHiddenFilesCheckBox = new QCheckBox(advancedGroupBox);
        syncHiddenFilesCheckBox->setObjectName("syncHiddenFilesCheckBox");

        verticalLayout_6->addWidget(syncHiddenFilesCheckBox);

        crashreporterCheckBox = new QCheckBox(advancedGroupBox);
        crashreporterCheckBox->setObjectName("crashreporterCheckBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(crashreporterCheckBox->sizePolicy().hasHeightForWidth());
        crashreporterCheckBox->setSizePolicy(sizePolicy2);

        verticalLayout_6->addWidget(crashreporterCheckBox);

        moveToTrashCheckBox = new QCheckBox(advancedGroupBox);
        moveToTrashCheckBox->setObjectName("moveToTrashCheckBox");

        verticalLayout_6->addWidget(moveToTrashCheckBox);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        ignoredFilesButton = new QPushButton(advancedGroupBox);
        ignoredFilesButton->setObjectName("ignoredFilesButton");

        horizontalLayout_4->addWidget(ignoredFilesButton);

        logSettingsButton = new QPushButton(advancedGroupBox);
        logSettingsButton->setObjectName("logSettingsButton");

        horizontalLayout_4->addWidget(logSettingsButton);

        spacer_3 = new QSpacerItem(555, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(spacer_3);


        verticalLayout_6->addLayout(horizontalLayout_4);


        horizontalLayout_5->addLayout(verticalLayout_6);


        verticalLayout_5->addWidget(advancedGroupBox);

        groupBox = new QGroupBox(scrollAreaWidgetContents_2);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        widget = new NetworkSettings(groupBox);
        widget->setObjectName("widget");
        widget->setFocusPolicy(Qt::StrongFocus);

        verticalLayout_4->addWidget(widget);


        verticalLayout_5->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_3->addWidget(scrollArea);

        versionInfoWidget = new QWidget(OCC__GeneralSettings);
        versionInfoWidget->setObjectName("versionInfoWidget");
        horizontalLayout_2 = new QHBoxLayout(versionInfoWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        spacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(spacer);

        copyrightLabel = new QLabel(versionInfoWidget);
        copyrightLabel->setObjectName("copyrightLabel");
        copyrightLabel->setCursor(QCursor(Qt::CursorShape::ArrowCursor));
        copyrightLabel->setText(QString::fromUtf8("Copyright ownCloud GmbH (A Kiteworks Company)"));
        copyrightLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout_2->addWidget(copyrightLabel);

        about_pushButton = new QPushButton(versionInfoWidget);
        about_pushButton->setObjectName("about_pushButton");

        horizontalLayout_2->addWidget(about_pushButton);


        verticalLayout_3->addWidget(versionInfoWidget);

#if QT_CONFIG(shortcut)
        languageLabel->setBuddy(languageDropdown);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(autostartCheckBox, desktopNotificationsCheckBox);
        QWidget::setTabOrder(desktopNotificationsCheckBox, monoIconsCheckBox);
        QWidget::setTabOrder(monoIconsCheckBox, languageDropdown);
        QWidget::setTabOrder(languageDropdown, syncHiddenFilesCheckBox);
        QWidget::setTabOrder(syncHiddenFilesCheckBox, crashreporterCheckBox);
        QWidget::setTabOrder(crashreporterCheckBox, moveToTrashCheckBox);
        QWidget::setTabOrder(moveToTrashCheckBox, ignoredFilesButton);
        QWidget::setTabOrder(ignoredFilesButton, logSettingsButton);
        QWidget::setTabOrder(logSettingsButton, widget);
        QWidget::setTabOrder(widget, scrollArea);
        QWidget::setTabOrder(scrollArea, about_pushButton);

        retranslateUi(OCC__GeneralSettings);

        QMetaObject::connectSlotsByName(OCC__GeneralSettings);
    } // setupUi

    void retranslateUi(QWidget *OCC__GeneralSettings)
    {
        OCC__GeneralSettings->setWindowTitle(QCoreApplication::translate("OCC::GeneralSettings", "Form", nullptr));
        generalGroupBox->setTitle(QCoreApplication::translate("OCC::GeneralSettings", "General Settings", nullptr));
        monoIconsCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Use Monochrome Icons in the system tray", nullptr));
        desktopNotificationsCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Show Desktop Notifications", nullptr));
        autostartCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Start on Login", nullptr));
        languageLabel->setText(QCoreApplication::translate("OCC::GeneralSettings", "Language", nullptr));
#if QT_CONFIG(accessibility)
        languageDropdown->setAccessibleName(QCoreApplication::translate("OCC::GeneralSettings", "Language selector", nullptr));
#endif // QT_CONFIG(accessibility)
        languageDropdown->setCurrentText(QString());
        advancedGroupBox->setTitle(QCoreApplication::translate("OCC::GeneralSettings", "Advanced", nullptr));
        syncHiddenFilesCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Sync hidden files", nullptr));
        crashreporterCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Show crash reporter", nullptr));
        moveToTrashCheckBox->setText(QCoreApplication::translate("OCC::GeneralSettings", "Move remotely deleted files to the local trash bin instead of deleting them", nullptr));
        ignoredFilesButton->setText(QCoreApplication::translate("OCC::GeneralSettings", "Edit Ignored Files", nullptr));
        logSettingsButton->setText(QCoreApplication::translate("OCC::GeneralSettings", "Log Settings", nullptr));
        groupBox->setTitle(QCoreApplication::translate("OCC::GeneralSettings", "Network", nullptr));
        about_pushButton->setText(QCoreApplication::translate("OCC::GeneralSettings", "About", nullptr));
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class GeneralSettings: public Ui_GeneralSettings {};
} // namespace Ui
} // namespace OCC

#endif // UI_GENERALSETTINGS_H
